#!/usr/bin/env python
import os
from copy import deepcopy
from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Routines import FileRoutines, SequenceRoutines, MatplotlibRoutines, DrawingRoutines
from RouToolPa.Collections.General import IdList, SynDict
from RouToolPa.Tools.LinuxTools import CGAS



class AUGUSTUS(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "augustus", path=path, max_threads=max_threads)

    def parse_options(self, species, genome_file="", strand="both", gene_model=None, output_gff3=True,
                      other_options="", config_dir=None, use_softmasking=None, hints_file=None,
                      extrinsicCfgFile=None, predict_UTR=None, min_intron_len=None):

        """
            AUGUSTUS (3.0.2) is a gene prediction tool for eukaryotes
        written by Mario Stanke (mstanke@gwdg.de) and Oliver Keller (keller@cs.uni-goettingen.de).

        usage:
        augustus [parameters] --species=SPECIES queryfilename

        'queryfilename' is the filename (including relative path) to the file containing the query sequence(s)
        in fasta format.

        SPECIES is an identifier for the species. Use --species=help to see a list.

        parameters:
        --strand=both, --strand=forward or --strand=backward
        --genemodel=partial, --genemodel=intronless, --genemodel=complete, --genemodel=atleastone or --genemodel=exactlyone
          partial      : allow prediction of incomplete genes at the sequence boundaries (default)
          intronless   : only predict single-exon genes like in prokaryotes and some eukaryotes
          complete     : only predict complete genes
          atleastone   : predict at least one complete gene
          exactlyone   : predict exactly one complete gene
        --singlestrand=true
          predict genes independently on each strand, allow overlapping genes on opposite strands
          This option is turned off by default.
        --hintsfile=hintsfilename
          When this option is used the prediction considering hints (extrinsic information) is turned on.
          hintsfilename contains the hints in gff format.
        --AUGUSTUS_CONFIG_PATH=path
          path to config directory (if not specified as environment variable)
        --alternatives-from-evidence=true/false
          report alternative transcripts when they are suggested by hints
        --alternatives-from-sampling=true/false
          report alternative transcripts generated through probabilistic sampling
        --sample=n
        --minexonintronprob=p
        --minmeanexonintronprob=p
        --maxtracks=n
          For a description of these parameters see section 4 of README.TXT.
        --proteinprofile=filename
          When this option is used the prediction will consider the protein profile provided as parameter.
          The protein profile extension is described in section 7 of README.TXT.
        --progress=true
          show a progressmeter
        --gff3=on/off
          output in gff3 format
        --predictionStart=A, --predictionEnd=B
          A and B define the range of the sequence for which predictions should be found.
        --UTR=on/off
          predict the untranslated regions in addition to the coding sequence. This currently works only for a subset of species.
        --noInFrameStop=true/false
          Do not report transcripts with in-frame stop codons. Otherwise, intron-spanning stop codons could occur. Default: false
        --noprediction=true/false
          If true and input is in genbank format, no prediction is made. Useful for getting the annotated protein sequences.
        --uniqueGeneId=true/false
          If true, output gene identifyers like this: seqname.gN

        For a complete list of parameters, type "augustus --paramlist".
        An exhaustive description can be found in the file README.TXT.
            """

        options = " --uniqueGeneId=true"
        options += " --extrinsicCfgFile=%s" % extrinsicCfgFile if extrinsicCfgFile else ""
        options += " --softmasking=1" if use_softmasking else ""
        options += " --UTR=on" if predict_UTR else ""
        options += " --hintsfile=%s" % hints_file if hints_file else ""
        options += (" %s" % other_options) if other_options else ""
        options += " --strand=%s" % strand
        options += (" --genemodel=%s" % gene_model) if gene_model else ""
        options += " --gff3=%s" % ("on" if output_gff3 else "no")
        options += " --species=%s" % species
        options += (" --AUGUSTUS_CONFIG_PATH=%s" % config_dir) if config_dir else ""
        options += " %s" % genome_file
        options += (" --min_intron_len=%i" % min_intron_len) if min_intron_len else ""

        return options

    def predict(self, species, genome_file, output, strand=None, gene_model=None, output_gff3=True,
                other_options="", use_softmasking=None, hints_file=None, extrinsicCfgFile=None,
                predict_UTR=None, min_intron_len=None):
        options = self.parse_options(species, genome_file=genome_file, strand=strand, gene_model=gene_model,
                                     output_gff3=output_gff3, other_options=other_options,
                                     use_softmasking=use_softmasking, hints_file=hints_file,
                                     extrinsicCfgFile=extrinsicCfgFile, predict_UTR=predict_UTR,
                                     min_intron_len=min_intron_len)
        options += " > %s" % output
        self.execute(options)

    def parallel_predict(self, species, genome_file, output, strand="both", gene_model=None, output_gff3=True,
                         other_options="", split_dir="splited_input", splited_output_dir="splited_output_dir",
                         config_dir=None, combine_output_to_single_file=True, use_softmasking=None, hints_file=None,
                         extrinsicCfgFile=None, predict_UTR=None, external_process_pool=None,
                         async_run=False, min_intron_len=None, parsing_mode="parse"):
        common_options = self.parse_options(species, genome_file="", strand=strand, gene_model=gene_model,
                                            output_gff3=output_gff3, other_options=other_options,
                                            config_dir=config_dir, use_softmasking=use_softmasking,
                                            hints_file=hints_file, extrinsicCfgFile=extrinsicCfgFile,
                                            predict_UTR=predict_UTR, min_intron_len=min_intron_len)

        splited_dir = FileRoutines.check_path(split_dir)
        splited_out_dir = FileRoutines.check_path(splited_output_dir)
        FileRoutines.safe_mkdir(splited_dir)
        FileRoutines.safe_mkdir(splited_out_dir)

        self.split_fasta_by_seq_len(genome_file, splited_dir, parsing_mode=parsing_mode)

        input_list_of_files = sorted(os.listdir(splited_dir))
        list_of_output_files = []
        options_list = []
        for filename in input_list_of_files:
            input_file = "%s%s" % (splited_dir, filename)
            output_file = "%s%s.gff" % (splited_out_dir, filename)
            list_of_output_files.append(output_file)
            options = common_options

            options += " %s" % input_file
            options += " > %s" % output_file
            options_list.append(options)

        self.parallel_execute(options_list, external_process_pool=external_process_pool, async_run=async_run)

        if combine_output_to_single_file:
            CGAS.cat(list_of_output_files, output=output)

    def extract_proteins_from_output(self, augustus_output, protein_output, evidence_stats_file=None,
                                     supported_by_hints_file=None, complete_proteins_id_file=None, id_prefix="p."):
        if evidence_stats_file:
            ev_fd = open(evidence_stats_file, "w")
            ev_fd.write("#gene_id\ttranscript_id\tsupported_fraction\tcds_support\tintron_support\t")
            ev_fd.write("5'UTR_support\t3'UTR_support\tincompatible_hints_groups\tprotein_length\n")

        if evidence_stats_file:
            sup_fd = open(supported_by_hints_file, "w")
            sup_fd.write("#gene_id\ttranscript_id\tsupported_fraction\tcds_support\tintron_support\t")
            sup_fd.write("5'UTR_support\t3'UTR_support\tincompatible_hints_groups\tprotein_length\n")

        if complete_proteins_id_file:
            complete_fd = open(complete_proteins_id_file, "w")

        with open(protein_output, "w") as out_fd:
            with open(augustus_output, "r") as in_fd:
                for line in in_fd:
                    if line[:12] == "# start gene":
                        gene = line.strip().split()[-1]
                    elif "\ttranscript\t" in line:
                        transcript_id = line.split("\t")[8].split(";")[0].split("=")[1]
                        start_presence = False
                        stop_presence = False
                        #out_fd.write(">%s%s\t gene=%s\n" % (id_prefix, transcript_id, gene))
                    elif "\tstart_codon\t" in line:
                        start_presence = True
                    elif "\tstop_codon\t" in line:
                        stop_presence = True
                    elif "# protein sequence" in line:
                        protein = line.strip().split("[")[-1]
                        if "]" in protein:
                            protein = protein.split("]")[0]
                        else:
                            while True:
                                part = in_fd.next().split()[-1]
                                if "]" in part:
                                    protein += part.split("]")[0]
                                    break
                                else:
                                    protein += part
                        if complete_proteins_id_file:
                            #print "AAAAA"
                            #print (start_presence, stop_presence)
                            if start_presence and stop_presence:
                                complete_fd.write("%s%s\n" % (id_prefix, transcript_id))

                        out_fd.write(">%s%s\t gene=%s start_presence=%s stop_presence=%s\n" % (id_prefix, transcript_id,
                                                                                               gene,
                                                                                               str(start_presence),
                                                                                               str(stop_presence)))
                        out_fd.write(protein)
                        protein_len = len(protein)
                        out_fd.write("\n")

                    elif evidence_stats_file or supported_by_hints_file:
                        if line[:17] == "# % of transcript":
                            supported_fraction = line.strip().split()[-1]
                            while True:
                                tmp_line = in_fd.next()
                                if tmp_line[:12] == "# CDS exons:":
                                    cds_support = tmp_line.strip().split()[-1]
                                elif tmp_line[:14] == "# CDS introns:":
                                    introns_support = tmp_line.strip().split()[-1]
                                elif tmp_line[:13] == "# 5'UTR exons":
                                    five_utr_support = tmp_line.strip().split()[-1]
                                elif tmp_line[:13] == "# 3'UTR exons":
                                    three_introns_support = tmp_line.strip().split()[-1]
                                elif tmp_line[:27] == "# incompatible hint groups:":
                                    incompatible_hint_groups = tmp_line.strip().split()[-1]
                                    if evidence_stats_file:
                                        ev_fd.write("%s\t%s\t%s\t" % (gene, transcript_id, supported_fraction))
                                        ev_fd.write("%s\t%s\t%s\t%s\t%s\t%i\n" % (cds_support, introns_support,
                                                                                  five_utr_support,
                                                                                  three_introns_support,
                                                                                  incompatible_hint_groups,
                                                                                  protein_len))
                                    if supported_by_hints_file and (float(supported_fraction) > 0):
                                        sup_fd.write("%s\t%s\t%s\t" % (gene, transcript_id, supported_fraction))
                                        sup_fd.write("%s\t%s\t%s\t%s\t%s\t%i\n" % (cds_support, introns_support,
                                                                                   five_utr_support,
                                                                                   three_introns_support,
                                                                                   incompatible_hint_groups,
                                                                                   protein_len))

                                    break

        if evidence_stats_file:
            ev_fd.close()

        self.extract_longest_isoforms(evidence_stats_file, "%s.longest_pep" % evidence_stats_file,
                                      minimum_supported_fraction=0)
        SequenceRoutines.extract_sequence_by_ids(protein_output,
                                                 "%s.longest_pep.ids" % evidence_stats_file,
                                                 "%s.longest_pep.pep" % evidence_stats_file)

        if supported_by_hints_file:
            supported_by_hints_longest_pep_evidence = "%s.longest_pep" % supported_by_hints_file
            supported_by_hints_longest_pep = "%s.longest_pep.pep" % supported_by_hints_file
            supported_by_hints_longest_pep_ids = "%s.longest_pep.ids" % supported_by_hints_file
            self.extract_longest_isoforms(evidence_stats_file, supported_by_hints_longest_pep_evidence,
                                          minimum_supported_fraction=0.00001)
            SequenceRoutines.extract_sequence_by_ids(protein_output,
                                                     supported_by_hints_longest_pep_ids,
                                                     supported_by_hints_longest_pep)

        evidence_files = (evidence_stats_file,
                          "%s.longest_pep" % evidence_stats_file,
                          "%s.longest_pep" % supported_by_hints_file) if supported_by_hints_file else \
                          (evidence_stats_file,)
        for evidence_file in evidence_files:
            print ("Drawing transcript support distribution for %s" % evidence_file)
            MatplotlibRoutines.percent_histogram_from_file(evidence_file,
                                                           evidence_file,
                                                           column_list=(2,), separator=None,
                                                           comments="#", n_bins=20,
                                                           title="Transcript support by hints",
                                                           xlabel="%%", ylabel="Number",
                                                           extensions=["svg", "png"],
                                                           legend_location="upper center",
                                                           stats_as_legend=True)

    @staticmethod
    def extract_evidence_by_ids(evidence_file, id_file, output_evidence_file, mode="transcript"):
        # possible modes: transcript, gene
        ids = IdList()
        ids.read(id_file, comments_prefix="#")

        column_id = 0 if mode == "gene" else 1

        with open(evidence_file, "r") as ev_fd:
            with open(output_evidence_file, "w") as out_fd:
                for line in ev_fd:
                    if line[0] == "#":
                        out_fd.write(line)
                        continue

                    entry_id = line.split("\t")[column_id]
                    if entry_id in ids:
                        out_fd.write(line)

    @staticmethod
    def extract_longest_isoforms(evidence_file, filtered_evidence_file, minimum_supported_fraction=0):
        longest_id_file = "%s.ids" % filtered_evidence_file
        ev_fd = open(evidence_file, "r")
        with open(longest_id_file, "w") as id_fd:
                with open(filtered_evidence_file, "w") as filtered_ev_fd:
                    filtered_ev_fd.write(ev_fd.readline())
                    for line in ev_fd:
                        prev_line = line
                        line_list = prev_line.strip().split("\t")
                        if float(line_list[2]) < minimum_supported_fraction:
                            continue
                        prev_gene = line_list[0]
                        prev_transcript = line_list[1]
                        prev_pep_len = line_list[-1]

                        break
                    for line in ev_fd:

                        line_list = line.strip().split("\t")
                        if float(line_list[2]) < minimum_supported_fraction:
                            continue
                        gene = line_list[0]
                        transcript = line_list[1]
                        pep_len = line_list[-1]
                        if gene == prev_gene:
                            if pep_len > prev_pep_len:
                                prev_line = line
                                prev_gene = gene
                                prev_transcript = transcript
                                prev_pep_len = pep_len
                            continue
                        id_fd.write("%s\n" % prev_transcript)
                        filtered_ev_fd.write(prev_line)
                        prev_line = line
                        prev_gene = gene
                        prev_transcript = transcript
                        prev_pep_len = pep_len
                    # if script breaks here it means that prediction was made without hints or all genes doesnt have hint support
                    id_fd.write("%s\n" % prev_transcript)
        ev_fd.close()

    @staticmethod
    def extract_CDS_annotations_from_output(augustus_output, CDS_output):
        CGAS.grep("'\\tCDS\\t'", augustus_output, output=CDS_output, use_regexp=True)

    @staticmethod
    def extract_gene_ids_from_output(augustus_output, all_genes_output):
        CGAS.cgas(augustus_output, grep_pattern="'\\tgene\\t'", sed_string="'s/.*ID=//'", output=all_genes_output,
                  grep_use_regexp=True)

    def exonerate_to_hints(self, exonerate_gff, hint_gff, priority=None, min_intron_len=None, max_intron_len=None,
                           CDS_part_cutoff=None, source=None, with_utrs=False):

        options = " --in=%s" % exonerate_gff
        options += " --out=%s" % hint_gff
        options += " --priority=%i" % priority if priority else ""
        options += " --minintronlen=%i" % min_intron_len if min_intron_len else ""
        options += " --maxintronlen=%i" % max_intron_len if max_intron_len else ""
        options += " --CDSpart_cutoff=%i" % CDS_part_cutoff if CDS_part_cutoff is not None else ""
        options += " --source=%s" % source if source else ""

        self.execute(options, cmd="exonerate2hints_with_utrs.pl" if with_utrs else "exonerate2hints.pl")

    def join_multiple_hints(self, in_file, out_file, separator=","):
        options = " > %s" % out_file
        # sorting of hints is necessary before merging
        cmd = "cat %s | sort -n -k 4,4 | sort -s -n -k 5,5 | sort -s -k 3,3 | sort -s -k 1,1 | join_mult_hints.pl" % (" ".join(in_file.split(separator) if isinstance(in_file, str) else in_file))
        self.execute(options, cmd=cmd)

    def assign_synonyms_to_features_from_augustus_gff(self, input_gff, output_file, prefix,
                                                      number_of_digits_in_number=8, feature_type="gene"):
        options = " > %s" % output_file
        cmd = """grep -P "\\t%s\\t" %s | sed 's/.*ID=//;s/;.*//' | awk -F'\\t' 'BEGIN {NUMBER=1};{printf "%%s\\t%s%%0%ii\\n",$1,NUMBER; NUMBER=NUMBER+1;}'""" % (feature_type, input_gff, prefix, number_of_digits_in_number)

        self.execute(options, cmd=cmd)

    def assign_synonyms_to_annotations_from_augustus_gff(self, input_gff, output_prefix, species_prefix,
                                                         number_of_digits_in_number=8):

        for feature in "gene", "transcript":
            output_file = "%s.%s.syn" % (output_prefix, feature)
            feature_prefix = "%s%s" % (species_prefix.upper(), "G" if feature == "gene" else "T")
            options = " > %s" % output_file
            cmd = """grep -P "\\t%s\\t" %s | sed 's/.*ID=//;s/;.*//' | awk -F'\\t' 'BEGIN {NUMBER=1};{printf "%%s\\t%s%%0%ii\\n",$1,NUMBER; NUMBER=NUMBER+1;}'""" % (feature, input_gff, feature_prefix, number_of_digits_in_number)
            self.execute(options, cmd=cmd)

        options = " %s.transcript.syn > %s.protein.syn" % (output_prefix, output_prefix)

        cmd = "sed 's/%sT/%sP/'" % (species_prefix, species_prefix)

        self.execute(options, cmd=cmd)

        options = " %s.transcript.syn > %s.cds.syn" % (output_prefix, output_prefix)
        cmd = "sed 's/%sT/%sC/;s/\\t/.cds\\t/'" % (species_prefix, species_prefix)
        self.execute(options, cmd=cmd)

    @staticmethod
    def replace_augustus_ids_by_syn(augustus_gff, output_gff, genes_syn_file, transcripts_syn_file,
                                    cds_syn_file=None):

        genes_syn_dict = SynDict()
        genes_syn_dict.read(genes_syn_file, comments_prefix="#")
        transcripts_syn_dict = SynDict()
        transcripts_syn_dict.read(transcripts_syn_file, comments_prefix="#")
        cds_syn_dict = SynDict()
        if cds_syn_file:
            cds_syn_dict.read(cds_syn_file, comments_prefix="#")
        with open(augustus_gff, "r") as in_fd:
            with open(output_gff, "w") as out_fd:
                for line in in_fd:
                    tmp = line.strip()
                    if len(tmp) < 13:
                        out_fd.write(line)
                        continue
                    if tmp[:12] != "# start gene":
                        out_fd.write(line)
                        continue
                    augustus_gene_id = tmp.split(" ")[-1]
                    gene_syn_id = genes_syn_dict[augustus_gene_id]
                    augustus_transcript_id = ""
                    augustus_transcript_parent = ""
                    out_fd.write("# start gene %s\n" % gene_syn_id)
                    tmp = in_fd.next().strip()
                    while True:
                        while tmp[0] != "#":
                            tmp_list = tmp.split("\t")
                            feature_type = tmp_list[2]
                            edited_str = "\t".join(tmp_list[:-1])
                            info_field_list = tmp_list[-1].split(";")
                            if feature_type == "gene":
                                edited_str += "\tID=%s\n" % gene_syn_id
                            elif feature_type == "transcript":
                                for entry in info_field_list:
                                    if "ID" in entry:
                                        augustus_transcript_id = entry.split("=")[-1]
                                        transcript_syn_id = transcripts_syn_dict[augustus_transcript_id]
                                    if "Parent" in entry:
                                        augustus_transcript_parent = entry.split("=")[-1]
                                        if augustus_transcript_parent != augustus_gene_id:
                                            raise ValueError("Transcript parent id and gene id are not same!")
                                edited_str += "\tID=%s;Parent=%s\n" % (transcript_syn_id, gene_syn_id)
                            elif feature_type == "CDS":
                                for entry in info_field_list:
                                    if "ID" in entry:
                                        augustus_cds_id = entry.split("=")[-1]
                                        cds_syn_id = cds_syn_dict[augustus_cds_id] if cds_syn_dict else "%s.cds" % transcripts_syn_dict[augustus_cds_id[:-4]]
                                    if "Parent" in entry:
                                        augustus_cds_parent = entry.split("=")[-1]
                                        if augustus_cds_parent != augustus_transcript_id:
                                            raise ValueError("CDS parent id and transcript id are not same!")
                                edited_str += "\tID=%s;Parent=%s\n" % (cds_syn_id, transcript_syn_id)
                            elif (feature_type == "stop_codon") or (feature_type == "start_codon"):
                                for entry in info_field_list:
                                    if "Parent" in entry:
                                        augustus_feature_parent = entry.split("=")[-1]
                                        if augustus_feature_parent != augustus_transcript_id:
                                            raise ValueError("Feature parent id and transcript id are not same!")
                                edited_str += "\tParent=%s\n" % (transcript_syn_id)
                            else:
                                edited_str = tmp

                            out_fd.write(edited_str)
                            tmp = in_fd.next().strip()
                        while tmp[0] == "#":
                            if "# end gene" in tmp:
                                break
                            out_fd.write(tmp + "\n")
                            tmp = in_fd.next().strip()
                        if "# end gene" in tmp:
                                break
                    out_fd.write("# end gene %s\n" % gene_syn_id)

    @staticmethod
    def replace_augustus_ids(augustus_gff, output_prefix, species_prefix=None, number_of_digits_in_id=8):

        output_gff = "%s.renamed.gff" % output_prefix
        genes_syn_file = "%s.gene.syn" % output_prefix
        transcripts_syn_file = "%s.transcript.syn" % output_prefix
        cds_syn_file = "%s.cds.syn" % output_prefix
        genes_syn_dict = SynDict()
        transcripts_syn_dict = SynDict()
        cds_syn_dict = SynDict()
        gene_counter = 0
        gene_id_template = "%sG%%0%ii" % (species_prefix, number_of_digits_in_id)
        transcripts_counter = 0
        transcript_id_template = "%sT%%0%ii" % (species_prefix, number_of_digits_in_id)
        cds_counter = 0
        cds_id_template = "%sC%%0%ii" % (species_prefix, number_of_digits_in_id)
        with open(augustus_gff, "r") as in_fd:
            with open(output_gff, "w") as out_fd:
                for line in in_fd:
                    tmp = line.strip()
                    if len(tmp) < 13:
                        out_fd.write(line)
                        continue
                    if tmp[:12] != "# start gene":
                        out_fd.write(line)
                        continue
                    augustus_gene_id = tmp.split(" ")[-1]
                    gene_counter += 1

                    gene_syn_id = gene_id_template % gene_counter
                    genes_syn_dict[augustus_gene_id] = gene_syn_id
                    augustus_transcript_id = ""
                    augustus_transcript_parent = ""
                    out_fd.write("# start gene %s\n" % gene_syn_id)
                    tmp = in_fd.next().strip()
                    while True:
                        while tmp[0] != "#":
                            tmp_list = tmp.split("\t")
                            feature_type = tmp_list[2]
                            edited_str = "\t".join(tmp_list[:-1])
                            info_field_list = tmp_list[-1].split(";")
                            if feature_type == "gene":
                                edited_str += "\tID=%s\n" % gene_syn_id
                            elif feature_type == "transcript":
                                for entry in info_field_list:
                                    if "ID" in entry:
                                        augustus_transcript_id = entry.split("=")[-1]
                                        if augustus_transcript_id not in transcripts_syn_dict:
                                            transcripts_counter += 1
                                            transcripts_syn_dict[augustus_transcript_id] = transcript_id_template % transcripts_counter
                                        transcript_syn_id = transcripts_syn_dict[augustus_transcript_id]
                                    if "Parent" in entry:
                                        augustus_transcript_parent = entry.split("=")[-1]
                                        if augustus_transcript_parent != augustus_gene_id:
                                            raise ValueError("Transcript parent id and gene id are not same!")
                                edited_str += "\tID=%s;Parent=%s\n" % (transcript_syn_id, gene_syn_id)
                            elif feature_type == "CDS":
                                for entry in info_field_list:
                                    if "ID" in entry:
                                        augustus_cds_id = entry.split("=")[-1]
                                        if augustus_cds_id not in cds_syn_dict:
                                            cds_counter += 1
                                            cds_syn_dict[augustus_cds_id] = cds_id_template % cds_counter
                                        cds_syn_id = cds_syn_dict[augustus_cds_id]
                                    if "Parent" in entry:
                                        augustus_cds_parent = entry.split("=")[-1]
                                        if augustus_cds_parent != augustus_transcript_id:
                                            raise ValueError("CDS parent id and transcript id are not same!")
                                edited_str += "\tID=%s;Parent=%s\n" % (cds_syn_id, transcript_syn_id)
                            elif (feature_type == "stop_codon") or (feature_type == "start_codon"):
                                for entry in info_field_list:
                                    if "Parent" in entry:
                                        augustus_feature_parent = entry.split("=")[-1]
                                        if augustus_feature_parent != augustus_transcript_id:
                                            raise ValueError("Feature parent id and transcript id are not same!")
                                edited_str += "\tParent=%s\n" % transcript_syn_id
                            else:
                                edited_str = tmp + "\n"

                            out_fd.write(edited_str)
                            tmp = in_fd.next().strip()
                        while tmp[0] == "#":
                            if "# end gene" in tmp:
                                break
                            out_fd.write(tmp + "\n")
                            tmp = in_fd.next().strip()
                        if "# end gene" in tmp:
                                break
                    out_fd.write("# end gene %s\n" % gene_syn_id)
        genes_syn_dict.write(genes_syn_file)
        transcripts_syn_dict.write(transcripts_syn_file)
        cds_syn_dict.write(cds_syn_file)

    @staticmethod
    def convert_star_junctions_to_intron_hints(star_junction_file, output_hint_gff, min_supporting_uniquely_mapped_reads=1,
                                               source="RNASEQ", priority=100):
        with open(star_junction_file, "r") as in_fd:
            with open(output_hint_gff, "w") as out_fd:
                for line in in_fd:
                    line_list = line.strip().split("\t")
                    chrom = line_list[0]
                    intron_start = line_list[1]
                    intron_end = line_list[2]
                    strand = line_list[3]
                    mult = line_list[6]
                    if int(mult) < min_supporting_uniquely_mapped_reads:
                        continue
                    # strand of intron will be detected automatically by AUGUSTUS
                    out_fd.write("%s\tSTAR\tintron\t%s\t%s\t0\t.\t.\tmult=%s;src=%s;pri=%i\n" % (chrom, intron_start,
                                                                                                 intron_end,
                                                                                                 mult, source,
                                                                                                 priority))

    @staticmethod
    def draw_evidence_figures(evidence_file, output_prefix, stats_as_legend=True):

        total_support_list = []

        cds_support_list = []
        intron_support_list = []

        with open(evidence_file, "r") as evid_fd:
            header = evid_fd.readline()
            for line in evid_fd:
                line_list = line.strip().split("\t")
                total_support_list.append(float(line_list[2]))

                cds_support_tmp = map(float, line_list[3].split("/"))
                cds_support_list.append(cds_support_tmp[0] / cds_support_tmp[1] * 100)

                intron_support_tmp = map(float, line_list[4].split("/"))
                if intron_support_tmp[1] == 0:
                    intron_support_list.append(-1)
                else:
                    intron_support_list.append(intron_support_tmp[0] / intron_support_tmp[1] * 100)

        print(len(total_support_list))
        print(len(cds_support_list))
        print(len(intron_support_list))
        print(intron_support_list.count(-1))
        DrawingRoutines.draw_heatmap_and_three_percent_histograms(total_support_list, cds_support_list,
                                                                  intron_support_list, output_prefix, figsize=(8, 8),
                                                                  extensions=("png", "svg"), stats_as_legend=stats_as_legend)

    @staticmethod
    def add_exon_lines_to_augustus_gff(augustus_gff, output_gff, number_of_digits_in_id=8, exon_id_prefix="EXON",
                                       new_exon_numering=False):
        transcript_cds_list = []
        exon_index = 1
        exon_prefix = "%sC%%0%ii" % (exon_id_prefix, number_of_digits_in_id)

        def line_handler(line, in_fd, out_fd):
            if line[0] == "#":
                    out_fd.write(line)
            else:
                if "\ttranscript\t" in line:
                    transcript_cds_list = []
                    start_codon_line = None
                    stop_codon_line = None

                    out_fd.write(line)

                    line = in_fd.next()

                    while ("\tstop_codon\t" in line) or ("\tstart_codon\t" in line) or ("\tCDS\t" in line):

                        if "\tstart_codon\t" in line:
                            start_codon_line = line
                        elif "\tstop_codon\t" in line:
                            stop_codon_line = line
                        else:
                            transcript_cds_list.append(line)
                        try:
                            line = in_fd.next()
                        except StopIteration:
                            line = None
                            break
                    #stop_codon_line = line

                    for CDS_line in transcript_cds_list:
                        CDS_line_list = CDS_line.strip().split("\t")
                        #print CDS_line
                        CDS_description_list = map(lambda string: string.split("="),
                                                   CDS_line_list[8].split(";"))
                        CDS_id = None

                        exon_description_list = deepcopy(CDS_description_list)
                        for i in range(len(exon_description_list)):
                            if exon_description_list[i][0] == "ID":
                                CDS_id = exon_description_list[i][1]

                                if new_exon_numering:
                                    exon_description_list[i][1] = exon_id_prefix % exon_index
                                    exon_index += 1
                                else:
                                    exon_description_list[i][1] = exon_prefix % int(CDS_id[-number_of_digits_in_id:])
                                break
                        else:
                            #print CDS_line
                            #print exon_description_list
                            raise ValueError("No ID was found for CDS")

                        exon_line = "%s\t%s\texon\t" % (CDS_line_list[0], CDS_line_list[1])
                        exon_line += "\t".join(CDS_line_list[3:8])

                        exon_description = ";".join(map(lambda s: "=".join(s), exon_description_list))
                        exon_line += "\t%s\n" % exon_description
                        out_fd.write(exon_line)
                    if not(start_codon_line is None):
                        out_fd.write(start_codon_line)

                    for CDS_line in transcript_cds_list:
                        out_fd.write(CDS_line)

                    if not(stop_codon_line is None):
                        out_fd.write(stop_codon_line)
                if line is None:
                    return 0

                if "\ttranscript\t" in line:
                    line_handler(line, in_fd, out_fd)
                out_fd.write(line)

        with open(augustus_gff, "r") as in_fd, open(output_gff, "w") as out_fd:
            for line in in_fd:
                line_handler(line, in_fd, out_fd)















#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Routines import NCBIRoutines



class Wget(Tool):
    def __init__(self, path="", max_threads=4):

        Tool.__init__(self, "wget", path=path, max_threads=max_threads)

    def parallel_download_from_sra(self, id_list):
        options_list = []
        for entry_id in id_list:
            ftp_path = NCBIRoutines.get_sra_ftp_path_from_id(entry_id)
            options_list.append("--no-host-directories -rc -t 500 %s" % ftp_path)

        self.parallel_execute(options_list)

        for filename in os.listdir(os.getcwd()):
            if ".sra" not in filename:
                continue
            self.safe_mkdir(filename[:-4])
            os.system("mv %s %s/" % (filename, filename[:-4]))

    def parallel_download(self, link_list):
        options_list = []
        for link in link_list:
            options_list.append("--no-host-directories -rc -t 500 -nv %s" % link)

        self.parallel_execute(options_list)

class Axel(Tool):
    def __init__(self, path="", max_threads=4):

        Tool.__init__(self, "axel", path=path, max_threads=max_threads)

    def parallel_download_from_sra(self, id_list, connections=4):

        options_list = []
        for entry_id in id_list:
            ftp_path = NCBIRoutines.get_sra_ftp_path_from_id(entry_id)
            options_list.append("-n %i %s" % (connections, ftp_path))

        self.parallel_execute(options_list)

        for filename in os.listdir(os.getcwd()):
            if ".sra" not in filename:
                continue
            self.safe_mkdir(filename[:-4])
            os.system("mv %s %s/" % (filename, filename[:-4]))

if __name__ == "__main__":
    pass



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'
from RouToolPa.Tools.Abstract import JavaTool
from RouToolPa.Tools.Samtools import SamtoolsV1



class MarkDuplicates(JavaTool):

    def __init__(self, java_path="", max_threads=4, jar_path="", max_memory="1g"):
        jar = "picard.jar MarkDuplicates"
        JavaTool.__init__(self, jar, java_path=java_path, max_threads=max_threads,
                          jar_path=jar_path, max_memory=max_memory)

    def parse_options(self, input_bam, output_bam, stat_file):

        options = " I= %s" % input_bam
        options += " O= %s" % output_bam
        options += " M= %s" % stat_file

        return options

    def run(self, input_bam, output_bam, stat_file):

        options = self.parse_options(input_bam, output_bam, stat_file)

        self.execute(options)

    def mkdup(self, input_bam, output_prefix):
        output_bam = "%s.bam" % output_prefix
        stat_file = "%s.stat" % output_prefix

        options = self.parse_options(input_bam, output_bam, stat_file)

        self.execute(options=options)
        SamtoolsV1.index(output_bam)

if __name__ == "__main__":
    pass


#!/usr/bin/env python

import os
from RouToolPa.Tools.Abstract import Tool




class Intersect(Tool):
    """
    Tool:    bedtools intersect (aka intersectBed)
    Version: v2.17.0
    Summary: Report overlaps between two feature files.

    Usage:   bedtools intersect [OPTIONS] -a <bed/gff/vcf> -b <bed/gff/vcf>

    Options:
        -abam	The A input file is in BAM format.  Output will be BAM as well.

        -ubam	Write uncompressed BAM output. Default writes compressed BAM.

        -bed	When using BAM input (-abam), write output as BED. The default
            is to write output in BAM when using -abam.

        -wa	Write the original entry in A for each overlap.

        -wb	Write the original entry in B for each overlap.
            - Useful for knowing _what_ A overlaps. Restricted by -f and -r.

        -loj	Perform a "left outer join". That is, for each feature in A
            report each overlap with B.  If no overlaps are found,
            report a NULL feature for B.

        -wo	Write the original A and B entries plus the number of base
            pairs of overlap between the two features.
            - Overlaps restricted by -f and -r.
              Only A features with overlap are reported.

        -wao	Write the original A and B entries plus the number of base
            pairs of overlap between the two features.
            - Overlapping features restricted by -f and -r.
              However, A features w/o overlap are also reported
              with a NULL B feature and overlap = 0.

        -u	Write the original A entry _once_ if _any_ overlaps found in B.
            - In other words, just report the fact >=1 hit was found.
            - Overlaps restricted by -f and -r.

        -c	For each entry in A, report the number of overlaps with B.
            - Reports 0 for A entries that have no overlap with B.
            - Overlaps restricted by -f and -r.

        -v	Only report those entries in A that have _no overlaps_ with B.
            - Similar to "grep -v" (an homage).

        -f	Minimum overlap required as a fraction of A.
            - Default is 1E-9 (i.e., 1bp).
            - FLOAT (e.g. 0.50)

        -r	Require that the fraction overlap be reciprocal for A and B.
            - In other words, if -f is 0.90 and -r is used, this requires
              that B overlap 90% of A and A _also_ overlaps 90% of B.

        -s	Require same strandedness.  That is, only report hits in B
            that overlap A on the _same_ strand.
            - By default, overlaps are reported without respect to strand.

        -S	Require different strandedness.  That is, only report hits in B
            that overlap A on the _opposite_ strand.
            - By default, overlaps are reported without respect to strand.

        -split	Treat "split" BAM or BED12 entries as distinct BED intervals.

        -sorted	Use the "chromsweep" algorithm for sorted (-k1,1 -k2,2n) input

        -header	Print the header from the A file prior to results.

    Notes:
        (1) When a BAM file is used for the A file, the alignment is retained if overlaps exist,
        and exlcuded if an overlap cannot be found.  If multiple overlaps exist, they are not
        reported, as we are only testing for one or more overlaps.
    """

    def __init__(self, path="", max_threads=1):
        Tool.__init__(self, "bedtools intersect", path=path, max_threads=max_threads)

    def intersect(self, a_file, b_file, out_file, method="-v"):

        options = " -a %s" % a_file
        options += " -b %s" % b_file
        options += " %s" % method
        options += " > %s" % out_file

        self.execute(options)

    def intersect_set(self, file_list, outfile, method="-v"):
        os.system("cat %s > temp0" % (file_list[0]))
        for i in range(1, len(file_list)):
            os.system("%sbedtools intersect %s -a %s -b %s > %s"
                      % (self.bedtools_dir, method, "temp%i" % (i-1), file_list[i], "temp%i" % i))
        os.system("cat temp%i > %s" % (i, outfile))
        os.system("rm -f %s" % " ".join(["temp%i" % x for x in range(0, i+1)]))



#!/usr/bin/env python
from RouToolPa.Tools.Abstract import Tool


class GetFasta(Tool):
    """
    Tool:    bedtools getfasta (aka fastaFromBed)
    Version: v2.17.0
    Summary: Extract DNA sequences into a fasta file based on feature coordinates.

    Usage:   bedtools getfasta [OPTIONS] -fi <fasta> -bed <bed/gff/vcf> -fo <fasta>

    Options:
        -fi	Input FASTA file
        -bed	BED/GFF/VCF file of ranges to extract from -fi
        -fo	Output file (can be FASTA or TAB-delimited)
        -name	Use the name field for the FASTA header
        -split	given BED12 fmt., extract and concatenate the sequencesfrom the BED "blocks" (e.g., exons)
        -tab	Write output in TAB delimited format.
            - Default is FASTA format.

        -s	Force strandedness. If the feature occupies the antisense,
            strand, the sequence will be reverse complemented.
            - By default, strand information is ignored.

    """

    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "bedtools getfasta", path=path, max_threads=max_threads)

    def get(self, bed_file, fasta_file, output_file, use_strand=False, use_name_field_for_fasta_header=False,
            output_tab=False):

        options = ""

        options += " -bed %s" % bed_file
        options += " -fi %s" % fasta_file
        options += " -fo %s" % output_file
        options += " -s" if use_strand else ""
        options += " -name" if use_name_field_for_fasta_header else ""
        options += " -tab" if output_tab else ""

        self.execute(options)




#!/usr/bin/env python

from collections import OrderedDict


class RecordTRF():
    def __init__(self, start, end, period, number_of_copies, consensus_pattern_size,
                 percent_of_matches, percent_of_indels, alignment_score, nucleotide_percent_list, entropy,
                 pattern=None, tandem_repeat=None, record_id=None):
        # TRF repeat string description:
        # Indices of the repeat relative to the start of the sequence.
        # Period size of the repeat.
        # Number of copies aligned with the consensus pattern.
        # Size of consensus pattern (may differ slightly from the period size).
        # Percent of matches between adjacent copies overall.
        # Percent of indels between adjacent copies overall.
        # Alignment score.
        # Percent composition for each of the four nucleotides.
        # Entropy measure based on percent composition.
        self.id = record_id
        self.start = start                                          # int
        self.end = end                                              # int
        self.period = period                                        # int
        self.number_of_copies = number_of_copies                    # float
        self.consensus_pattern_size = consensus_pattern_size        # int
        self.percent_of_matches = percent_of_matches                # int
        self.percent_of_indels = percent_of_indels                  # int
        self.alignment_score = alignment_score                      # int
        self.nucleotide_percent_list = nucleotide_percent_list      # list of four ints
        self.entropy = entropy                                      # float
        self.pattern = pattern                                      # str or None
        self.tandem_repeat = tandem_repeat                          # str or None

    def attributes_string(self):
        attributes_string = "ID=%s;" % str(self.id) if self.id else ""
        attributes_string += "Period=%i;N_copies=%.1f;Pattern=%s;Cons_pat_size=%i;Pers_matches=%i;Pers_indels=%i;Align_score=%i" \
                             % (self.period, self.number_of_copies, self.pattern, self.consensus_pattern_size,
                                self.percent_of_matches, self.percent_of_indels, self.alignment_score)
        nuc_composition = ",".join(map(lambda x: str(x), self.nucleotide_percent_list))

        attributes_string += ";Nuc_composition=%s;Entropy=%.2f" % (nuc_composition, self.entropy)

        return attributes_string

    def atributes_string_with_rep_seq(self):
        return self.attributes_string() + ";seq=%s" % self.tandem_repeat

    def gff_str_with_rep_seq(self):
        return "TRF\trepeat\t%i\t%i\t.\t.\t.\t%s" % (self.start, self.end, self.atributes_string_with_rep_seq())

    def gff_str(self):
        # seqid	source	type	start	end	score	strand	phase	attributes
        return "TRF\trepeat\t%i\t%i\t.\t.\t.\t%s" % (self.start, self.end, self.attributes_string())

    def simple_gff_str(self):
        # seqid	source	type	start	end	score	strand	phase	attributes
        attributes_string = "ID=%s;" % self.id if self.id else ""
        attributes_string += "Period=%i;N_copies=%.1f;Pattern=%s" % (self.period, self.number_of_copies, self.pattern)

        return "TRF\trepeat\t%i\t%i\t.\t.\t.\t%s" % (self.start, self.end, attributes_string)

    def table_str_short(self):

        return "%i\t%i\t%i\t%.1f\t%s\t%s" % (self.start, self.end, self.period, self.number_of_copies,
                                             self.pattern, self.tandem_repeat)

    def simple_bed_str(self):

        return "%i\t%i\t%s\t.\t." % (self.start, self.end, self.id)

    def table_str_long(self):

        return "%i\t%i\t%i\t%.1f\t%i\t%i\t%i\t%i\t%s\t%.2f\t%s\t%s" % (self.start, self.end, self.period,
                                                                       self.number_of_copies,
                                                                       self.consensus_pattern_size,
                                                                       self.percent_of_matches,
                                                                       self.percent_of_indels,
                                                                       self.alignment_score,
                                                                       ",".join(map(lambda x: str(x), self.nucleotide_percent_list)),
                                                                       self.entropy,
                                                                       self.pattern, self.tandem_repeat)


class CollectionTRF():

    def __init__(self, parameters=None, record_list=None, trf_file=None, from_file=True, add_ids=True,
                 record_id_prefix="TRF"):
        self.linkage_dict = None
        record_index = 0
        if from_file:
            self.records = {}
            for trf_filename in [trf_file] if isinstance(trf_file, str) else trf_file:
                with open(trf_filename, "r") as fd:
                    chrom = None
                    for line in fd:
                        #tmp = line.strip()
                        if line[0:8] == "Sequence":
                            chrom = line.strip().split()[1]
                        elif not chrom:
                            continue
                        elif line[0:10] == "Parameters":
                            self.parameters = list(map(lambda x: int(x), line.strip().split()[1:]))
                        elif line != "\n":
                            self._add_record(line, chrom, record_id="%s%i" % (record_id_prefix, record_index) if add_ids else None)
                            record_index += 1
                """
                tmp = next(fd)
                while tmp[0:8] != "Sequence":
                        tmp = next(fd)
                while True:
                    #line = fd.readline()
                    while tmp[0:8] != "Sequence":
                        tmp = next(fd)
                        #print tmp
                    chrom = tmp.strip().split()[1]
                    while tmp[0:10] != "Parameters":
                        tmp = next(fd)
                        #print tmp
                    print tmp
                    self.parameters = list(map(lambda x: int(x), tmp.strip().split()[1:]))
                    tmp = next(fd)
                    while tmp == "\n":
                        tmp = next(fd)
                    print tmp
                    while (tmp != "\n") and (tmp != "") and (tmp[0:8] != "Sequence"):
                        self._add_record(tmp, chrom)
                        try:
                         tmp = next(fd)
                        except StopIteration:
                            break
                """
        else:
            self.records = record_list
            self.parameters = parameters

    def _add_record(self, line, chrom, record_id=None):
        line_list = line.strip().split()
        start, end, period = list(map(lambda x: int(x), line_list[0:3]))
        consensus_pattern_size, percent_of_matches, percent_of_indels, alignment_score = \
            list(map(lambda x: int(x), line_list[4:8]))
        nucleotide_percent_list = list(map(lambda x: int(x), line_list[8:-3]))
        record = RecordTRF(start, end, period, float(line_list[3]), consensus_pattern_size,
                           percent_of_matches, percent_of_indels, alignment_score,
                           nucleotide_percent_list,
                           float(line_list[-3]),
                           pattern=line_list[-2], tandem_repeat=line_list[-1],
                           record_id=record_id)
        if chrom not in self.records:
            self.records[chrom] = []
        self.records[chrom].append(record)

    def write_gff_with_rep_seqs(self, out_file, write_chr_string=False):
        with open(out_file, "w") as out_fd:
            for chrom in self.records:
                if write_chr_string:
                    out_fd.write("#%s\n" % chrom)
                for record in self.records[chrom]:
                    out_fd.write("%s\t%s\n" % (chrom, record.gff_str_with_rep_seq()))

    def write_gff(self, out_file, write_chr_string=False):
        with open(out_file, "w") as out_fd:
            for chrom in self.records:
                if write_chr_string:
                    out_fd.write("#%s\n" % chrom)
                for record in self.records[chrom]:
                    out_fd.write("%s\t%s\n" % (chrom, record.gff_str()))

    def write_simple_gff(self, out_file, write_chr_string=False):
        with open(out_file, "w") as out_fd:
            for chrom in self.records:
                if write_chr_string:
                    out_fd.write("#%s\n" % chrom)
                for record in self.records[chrom]:
                    out_fd.write("%s\t%s\n" % (chrom, record.simple_gff_str()))

    def write(self, out_file):
        with open(out_file, "w") as out_fd:
            out_fd.write("#>chrom\n#\tstart\tend\tperiod\tnumber_of_copies\tpattern\ttandem_repeat\n")
            for chrom in self.records:
                out_fd.write(">%s\n" % chrom)
                for record in self.records[chrom]:
                    out_fd.write("\t%s\n" % record.table_str_short())

    def write_bed(self, out_file):
        with open(out_file, "w") as out_fd:
            out_fd.write("#chrom\tstart\tend\tid\tscore\tstrand\n")
            for chrom in self.records:
                for record in self.records[chrom]:
                    out_fd.write("%s\t%s\n" % (chrom, record.simple_bed_str()))

    def write_short_table(self, out_file):
        with open(out_file, "w") as out_fd:
            out_fd.write("#chrom\tstart\tend\tperiod\tnumber_of_copies\tpattern\ttandem_repeat\n")
            for chrom in self.records:
                for record in self.records[chrom]:
                    out_fd.write("%s\t%s\n" % (chrom, record.table_str_short()))

    def write_wide_table(self, out_file):
        with open(out_file, "w") as out_fd:
            out_fd.write("#chrom\tstart\tend\tperiod\tnumber_of_copies\tconsensus_pattern_size\tpercent_of_matches\tpercent_of_indels\talignment_score\tnucleotide_composition\tentropy\tpattern\ttandem_repeat\n")
            for chrom in self.records:
                for record in self.records[chrom]:
                    out_fd.write("%s\t%s\n" % (chrom, record.table_str_long()))

    def write_fasta(self, out_file):
        with open(out_file, "w") as out_fd:
            for chrom in self.records:
                for record in self.records[chrom]:
                    if record.id and record.tandem_repeat:
                        out_fd.write(">%s location:%s,%i,%i pattern:%s period:%i copies:%f matches,%%:%i "
                                     "indels,%%:%i align_score:%i entropy:%f\n%s\n" % (record.id,
                                                                                       chrom,
                                                                                       record.start,
                                                                                       record.end,
                                                                                       record.pattern,
                                                                                       record.period,
                                                                                       record.number_of_copies,
                                                                                       record.percent_of_matches,
                                                                                       record.percent_of_indels,
                                                                                       record.alignment_score,
                                                                                       record.entropy,
                                                                                       record.tandem_repeat))

    def get_id_based_dict(self):
        """
        returns following dictionary {trf_id: [chrom, start, end]}
        """
        id_based_dict = OrderedDict()
        for chrom in self.records:
            for record in self.records[chrom]:
                id_based_dict[record.id] = [chrom, record.start, record.end]

        return id_based_dict


if __name__ == "__main__":
    trf_coll = CollectionTRF(trf_file="/home/mahajrod/genetics/desaminases/data/LAN210_v0.10m/masking/TRF/LAN210_v0.10m_masked_repeatmasker.fasta.2.7.7.80.10.50.500.dat", from_file=True)
    trf_coll.write_gff("/home/mahajrod/genetics/desaminases/data/LAN210_v0.10m/masking/TRF/trf.gff")



#!/usr/bin/env python

__author__ = 'mahajrod'

from RouToolPa.Tools.Abstract import Tool


class BaseRecalibrator4(Tool):

    def __init__(self, max_threads=4, max_memory=None, timelog=None):
        Tool.__init__(self,
                      "gatk --java-options -Xmx%s BaseRecalibrator" % max_memory if max_memory else "gatk BaseRecalibrator",
                      max_threads=max_threads, max_memory=max_memory,
                      timelog=timelog)

    def get_recalibration_table(self, reference, alignment, output_table, known_sites_vcf, BQSR=None,
                                include_region_id_file=None, exclude_region_id_file=None):

        # TODO: add rest of  options
        options = ""
        options += " -R %s" % reference
        options += " -I %s" % alignment
        options += " -nct %i" % self.threads
        options += " -BQSR %s" % BQSR if BQSR else ""
        options += " -knownSites %s" % known_sites_vcf if isinstance(known_sites_vcf, str) else " -knownSites ".join(known_sites_vcf)
        options += " -L %s" % include_region_id_file if include_region_id_file else ""
        options += " -XL %s" % exclude_region_id_file if exclude_region_id_file else ""
        options += " -o %s" % output_table

        self.execute(options)



#!/usr/bin/env python
import os
from RouToolPa.Tools.Abstract import Tool


class RepeatModeler(Tool):
    def __init__(self, path="", max_threads=1):
        Tool.__init__(self, "RepeatModeler", path=path, max_threads=max_threads)

    def build_db(self, database_name, fasta_dir=None, file_with_filenames=None, engine="ncbi"):

        if (not fasta_dir) and (not file_with_filenames):
            raise ValueError("Neither fasta file/directory with fasta files or file with paths to fasta files were set")

        options = " -name %s" % database_name
        options += " -dir %s" % fasta_dir if fasta_dir else ""
        options += " -engine %s" % engine if engine else ""
        options += " -batch %s " % file_with_filenames if file_with_filenames else ""

        self.execute(options, cmd="BuildDatabase")

    def annotate_repeats(self, database, engine="ncbi", recover_dir=None):

        options = " -pa %i" % self.threads
        options += " -database %s" % database
        options += " -engine %s" % engine if engine else ""
        options += " -recoverDir %s " % recover_dir if recover_dir else ""

        self.execute(options)


#!/usr/bin/env python
import os
from RouToolPa.Tools.Abstract import Tool


class AnalyzeCovariates4(Tool):

    def __init__(self,  max_threads=4, max_memory=None, timelog=None):
        Tool.__init__(self,
                      "gatk --java-options -Xmx%s AnalyzeCovariates" % max_memory if max_memory else "gatk AnalyzeCovariates",
                      max_threads=max_threads, max_memory=max_memory,
                      timelog=timelog)

    # http://www.broadinstitute.org/gatk/gatkdocs/org_broadinstitute_sting_gatk_walkers_variantutils_CombineVariants.html

    def parse_options(self, reference, BQSR_table=None, output_plots=None, before_table=None, after_table=None,
                      ignoreLMT=False, csv_out=None): #, known_sites=None):

        options = " -R %s" % reference
        options += " -BQSR %s" % BQSR_table if BQSR_table else ""
        options += " -plots %s" % output_plots if output_plots else ""
        options += " -before %s" % before_table if before_table else ""
        options += " -after %s" % after_table if after_table else ""
        options += " -ignoreLMT" if ignoreLMT else ""
        options += " -csv %s" % csv_out if csv_out else ""

        #if known_sites:
        #    if isinstance(known_sites, str):
        #        options += " -knownSites %s" % known_sites
        #    else:
        #        for entry in known_sites:
        #            options += " -knownSites %s" % entry

        return options

    def plot_single_recall_table(self, reference, BQSR_table, output_plots, csv_out=None):
        """
        gatk AnalyzeCovariates \
          -R myrefernce.fasta \
          -BQSR myrecal.table \
          -plots BQSR.pdf
        """

        options = self.parse_options(reference, BQSR_table=BQSR_table, output_plots=output_plots, csv_out=csv_out)

        self.execute(options=options)

    def plot_two_recall_table(self, reference, before_table, after_table, output_plots, csv_out=None):
        """
        Plot before (first pass) and after (second pass) recalibration tables to compare them
         gatk AnalyzeCovariates \
              -R myrefernce.fasta \
              -before recal2.table \
              -after recal3.table \
              -plots recalQC.pdf
        """

        options = self.parse_options(reference, before_table=before_table, csv_out=csv_out,
                                     after_table=after_table, output_plots=output_plots)

        self.execute(options=options)

    def plot_three_recall_table(self, reference, BQSR_table, before_table, after_table, output_plots,
                                ignoreLMT=False, csv_out=None):
        """
        Plot up to three recalibration tables for comparison

         # You can ignore the before/after semantics completely if you like (if you do, add -ignoreLMT
         # to avoid a possible warning), but all tables must have been generated using the same parameters.

         gatk AnalyzeCovariates \
              -R myrefernce.fasta \
              -ignoreLMT \
              -BQSR recal1.table \   # you can discard any two
              -before recal2.table \
              -after recal3.table \
              -plots myrecals.pdf
        """

        options = self.parse_options(reference, BQSR_table=BQSR_table, before_table=before_table, csv_out=csv_out,
                                     after_table=after_table, output_plots=output_plots, ignoreLMT=ignoreLMT)

        self.execute(options=options)




#!/usr/bin/env python
import os
from copy import deepcopy
from collections import OrderedDict
from Bio import SeqIO
from RouToolPa.Routines.Sequence import SequenceRoutines
from RouToolPa.Collections.General import SynDict, IdSet, IdList




class SequenceClusterRoutines(SequenceRoutines):

    def __init__(self):
        SequenceRoutines.__init__(self)

    def read_cluster_files_from_dir(self, dir_with_cluster_files):
        cluster_files_list = sorted(os.listdir(dir_with_cluster_files))
        clusters_dict = OrderedDict()
        for filename in cluster_files_list:
            filepath = "%s%s" % (self.check_path(dir_with_cluster_files), filename)
            filename_list = self.split_filename(filepath)
            clusters_dict[filename_list[1]] = SynDict()
            clusters_dict[filename_list[1]].read(filepath, header=False, separator="\t", allow_repeats_of_key=False,
                                                 split_values=True, values_separator=",", key_index=0, value_index=1,
                                                 comments_prefix="#")
        return clusters_dict

    @staticmethod
    def get_cluster_names(clusters_dict, out_file=None, white_list_ids=None):
        cluster_names = IdSet()
        for species in clusters_dict:
            species_clusters = IdSet(clusters_dict[species].keys())
            cluster_names |= species_clusters
        if out_file:
            cluster_names.write(out_file)

        return cluster_names & IdSet(white_list_ids) if white_list_ids else cluster_names

    @staticmethod
    def get_sequence_names(clusters_dict, write_ids=False, out_prefix=None, white_list_ids=None):
        sequence_names_dict = SynDict()
        for species in clusters_dict:
            sequence_names_dict[species] = IdSet()
        for species in clusters_dict:
            for cluster_id in clusters_dict[species]:
                if white_list_ids:
                    if cluster_id not in white_list_ids:
                        continue
                sequence_names_dict[species] = sequence_names_dict[species] | IdSet(clusters_dict[species][cluster_id])
        if write_ids:
            for species in clusters_dict:
                out_file = "%s_%s.ids" % (out_prefix, species) if out_prefix else "%s.ids" % species
                sequence_names_dict[species].write(out_file)
        return sequence_names_dict

    @staticmethod
    def merge_clusters(clusters_dict, label_species="False", separator_for_labeling="_",
                       species_label_first=True):

        if species_label_first:
            label_sequence = lambda label, name: "%s%s%s" % (label, separator_for_labeling, name)
        else:
            label_sequence = lambda label, name: "%s%s%s" % (name, separator_for_labeling, label)
        if label_species:
            expression = label_sequence
        else:
            expression = lambda label, name: name

        merged_clusters = SynDict()
        for species in clusters_dict:
            for cluster in clusters_dict[species]:
                if cluster not in merged_clusters:
                    merged_clusters[cluster] = []
                for sequence_name in clusters_dict[species][cluster]:
                    merged_clusters[cluster].append(expression(species, sequence_name))

        return merged_clusters

    def merge_clusters_from_files(self, dir_with_cluster_files, output_file, label_species="False",
                                  separator_for_labeling="@", species_label_first=True):

        clusters_dict = self.read_cluster_files_from_dir(dir_with_cluster_files)
        merged_clusters = self.merge_clusters(clusters_dict, label_species=label_species,
                                              separator_for_labeling=separator_for_labeling,
                                              species_label_first=species_label_first)
        merged_clusters.write(output_file, splited_values=True)
        return merged_clusters

    def extract_monocluster_ids(self, clusters_dict, white_list_ids=None, out_file=None):
        """
        Extracts clusters with only one sequence in all species.
        """
        monocluster_ids = IdSet()
        cluster_names = self.get_cluster_names(clusters_dict)

        for cluster_name in cluster_names:
            for species in clusters_dict:
                if white_list_ids:
                    if cluster_name not in white_list_ids:
                        break
                if cluster_name not in clusters_dict[species]:
                    break
                if len(clusters_dict[species][cluster_name]) > 1:
                    break
            else:
                monocluster_ids.add(cluster_name)

        if out_file:
            monocluster_ids.write(out_file)

        return monocluster_ids

    def extract_monocluster_ids_from_file(self, dir_with_cluster_files, out_file, file_with_white_list_ids=None):
        # filenames are counted as species names
        white_list_ids = None
        if file_with_white_list_ids:
            white_list_ids = IdSet()
            white_list_ids.read(file_with_white_list_ids)
        clusters_dict = self.read_cluster_files_from_dir(dir_with_cluster_files)
        monoclusters = self.extract_monocluster_ids(clusters_dict, out_file=out_file, white_list_ids=white_list_ids)
        return monoclusters

    def extract_sequences_by_clusters(self, dir_with_cluster_files, dir_with_sequence_files, output_dir,
                                      file_with_white_list_cluster_ids=None, mode="families",
                                      sequence_file_extension="fasta", sequence_file_format="fasta",
                                      label_species=False, separator_for_labeling="@", species_label_first=True):
        """
        basenames of cluster and sequence files must be same

        mode:
            clusters - extract sequences from clusters in separate files,
            species - extract sequences from species to separate files
        """
        white_list_ids = None
        if file_with_white_list_cluster_ids:
            white_list_ids = IdSet()
            white_list_ids.read(file_with_white_list_cluster_ids)

        clusters_dict = self.read_cluster_files_from_dir(dir_with_cluster_files)
        cluster_names = self.get_cluster_names(clusters_dict, white_list_ids=white_list_ids)

        sequence_super_dict = OrderedDict()
        out_dir = self.check_path(output_dir)

        for species in clusters_dict:
            idx_file = "%s_tmp.idx" % species
            sequence_file = "%s%s.%s" % (self.check_path(dir_with_sequence_files), species,
                                         sequence_file_extension)
            sequence_super_dict[species] = SeqIO.index_db(idx_file, sequence_file, format=sequence_file_format)

        if mode == "species":
            seqeuence_names = self.get_sequence_names(clusters_dict, write_ids=False, out_prefix=None,
                                                      white_list_ids=white_list_ids)
            for species in seqeuence_names:
                out_file = "%s%s.%s" % (out_dir, species, sequence_file_extension)
                SeqIO.write(SequenceRoutines.record_by_id_generator(sequence_super_dict[species],
                                                                    seqeuence_names[species]),
                            out_file, format=sequence_file_format)
        elif mode == "families":

            def per_family_record_generator(seq_super_dict, clust_dict, cluster_id):
                if species_label_first:
                    label_sequence = lambda label, name: "%s%s%s" % (label, separator_for_labeling, name)
                else:
                    label_sequence = lambda label, name: "%s%s%s" % (name, separator_for_labeling, label)

                for species in seq_super_dict:
                    #print species, cluster_id
                    for record_id in clust_dict[species][cluster_id]:
                        if label_species:
                            record = deepcopy(seq_super_dict[species][record_id])
                            record.id = label_sequence(species, record_id)
                            yield record
                        else:
                            yield seq_super_dict[species][record_id]

            for cluster_name in cluster_names:
                out_file = "%s%s.%s" % (out_dir, cluster_name, sequence_file_extension)
                SeqIO.write(per_family_record_generator(sequence_super_dict, clusters_dict, cluster_name),
                            out_file, format=sequence_file_format)

        for species in clusters_dict:
            os.remove("%s_tmp.idx" % species)

    @staticmethod
    def rename_elements_in_clusters(clusters_file, syn_file, output_clusters_file,
                                    remove_clusters_with_not_renamed_elements=False,
                                    elements_with_absent_synonyms_file=None,
                                    syn_file_key_column_index=0,
                                    syn_file_value_column_index=1,
                                    syn_file_column_separator='\t'):
        syn_dict = SynDict()
        syn_dict.read(syn_file, comments_prefix="#", key_index=syn_file_key_column_index,
                      value_index=syn_file_value_column_index, separator=syn_file_column_separator)

        clusters_dict = SynDict()
        clusters_dict.read(clusters_file, split_values=True, values_separator=",", comments_prefix="#")

        output_clusters_dict = SynDict()

        absent_elements_dict = SynDict()

        for cluster in clusters_dict:
            renamed_element_list = []
            all_elements_were_renamed_flag = True
            for element in clusters_dict[cluster]:
                if element in syn_dict:
                    renamed_element_list.append(syn_dict[element])
                else:
                    if cluster not in absent_elements_dict:
                        absent_elements_dict[cluster] = [element]
                    else:
                        absent_elements_dict[cluster].append(element)
                    all_elements_were_renamed_flag = False
                    renamed_element_list.append(element)

            if (not remove_clusters_with_not_renamed_elements) or (remove_clusters_with_not_renamed_elements and all_elements_were_renamed_flag):
                output_clusters_dict[cluster] = renamed_element_list

        output_clusters_dict.write(output_clusters_file, splited_values=True)

        if elements_with_absent_synonyms_file:
            absent_elements_dict.write(elements_with_absent_synonyms_file, splited_values=True)

        return absent_elements_dict

    @staticmethod
    def check_absence_of_cluster_elements(fam_list, sequence_dict):
        absent_elements = []
        for element in fam_list:
            if element not in sequence_dict:
                absent_elements.append(element)
        return absent_elements

    def extract_sequences_from_selected_clusters(self, clusters_id_file, cluster_file, seq_file,
                                                 output_dir="./", seq_format="fasta",
                                                 out_prefix=None, create_dir_for_each_cluster=False,
                                                 skip_cluster_if_no_sequence_for_element=True,
                                                 parsing_mode="parse"):
        from RouToolPa.Routines import SequenceRoutines
        cluster_id_list = IdList()
        cluster_dict = SynDict()
        #print(pep_file)
        self.safe_mkdir(output_dir)
        out_dir = self.check_path(output_dir)
        create_directory_for_each_cluster = True if out_prefix else create_dir_for_each_cluster
        if clusters_id_file:
            cluster_id_list.read(clusters_id_file)
        cluster_dict.read(cluster_file, split_values=True, values_separator=",")

        protein_files = self.make_list_of_path_to_files(seq_file)

        protein_dict = self.parse_seq_file(protein_files, "index_db", format=seq_format, index_file="tmp.idx") if len(protein_files) > 1 else self.parse_seq_file(protein_files[0], parsing_mode, format=seq_format, index_file="tmp.idx") #SeqIO.index_db("tmp.idx", self.make_list_of_path_to_files(seq_file), format=seq_format)

        number_of_skipped_clusters = 0
        for fam_id in cluster_id_list if clusters_id_file else cluster_dict:

            if skip_cluster_if_no_sequence_for_element:
                absent_elements = self.check_absence_of_cluster_elements(cluster_dict[fam_id], protein_dict)
                if absent_elements:
                    print("Skipping cluster %s due to absent element(%s)" % (fam_id, ",".join(absent_elements)))
                    number_of_skipped_clusters += 1
                    continue

            if fam_id in cluster_dict:
                if create_directory_for_each_cluster:
                    fam_dir = "%s%s/" % (out_dir, fam_id)
                    self.safe_mkdir(fam_dir)
                    out_file = "%s%s.fasta" % (fam_dir, out_prefix if out_prefix else fam_id)
                else:
                    out_file = "%s/%s.fasta" % (out_dir, out_prefix if out_prefix else fam_id)

                SeqIO.write(SequenceRoutines.record_by_id_generator(protein_dict, cluster_dict[fam_id], verbose=True),
                            out_file, format=seq_format)

        if (len(protein_files) > 1) or (parsing_mode == "index_db"):
            os.remove("tmp.idx")
        print ("%i of %i clusters were skipped due to absent elements" % (number_of_skipped_clusters, len(cluster_dict)))

        return number_of_skipped_clusters

    @staticmethod
    def extract_clusters_by_element_ids(cluster_dict, element_id_list, mode="w"):
        """"
        mode: "w" - if elements from element_id_list are present in cluster extracts only that elements
              "a" - if elements from element_id_list are present in cluster extracts all elements
        """

        extracted_clusters = SynDict()
        for cluster in cluster_dict:
            extracted_elements = []
            if mode == "w":
                for element in cluster_dict[cluster]:
                    if element in element_id_list:
                        extracted_elements.append(element)
                if extracted_elements:
                    extracted_clusters[cluster] = extracted_elements
            elif mode == "a":
                for element in cluster_dict[cluster]:
                    if element in element_id_list:
                        extracted_clusters[cluster] = cluster_dict[cluster]
                        break

        return extracted_clusters

    def extract_clusters_by_element_ids_from_file(self, cluster_file, element_file, output_file, mode="w",
                                                  cluster_column=0, element_column=1, column_separator="\t",
                                                  element_separator=",", id_column=None):
        """"
        mode: "w" - if elements from element_id_list are present in cluster extracts only that elements
              "a" - if elements from element_id_list are present in cluster extracts all elements
        """
        cluster_dict = SynDict(filename=cluster_file, split_values=True, comments_prefix="#",
                               key_index=cluster_column, value_index=element_column,
                               separator=column_separator, values_separator=element_separator)

        element_id_list = IdList(filename=element_file, comments_prefix="#", column_number=id_column)
        extracted_clusters = self.extract_clusters_by_element_ids(cluster_dict, element_id_list, mode=mode)
        extracted_clusters.write(output_file, splited_values=True)

    @staticmethod
    def label_cluster_elements(cluster_dict, label, separator="@", label_position="first"):
        labeled_cluster_dict = SynDict()
        if label_position == "first":
            label_function = lambda s: "%s%s%s" % (label, separator, s)
        elif label_position == "last":
            label_function = lambda s: "%s%s%s" % (s, separator, label)

        for cluster in cluster_dict:
            labeled_cluster_dict[cluster] = []
            for element in cluster_dict[cluster]:
                labeled_cluster_dict[cluster].append(label_function(element))

        return labeled_cluster_dict

    def label_cluster_elements_from_file(self, input_file, label, output_file, separator="@", label_position="first",
                                         key_index=0, value_index=1):
        input_dict = SynDict(filename=input_file, split_values=True, comments_prefix="#",
                             key_index=key_index, value_index=value_index)

        output_dict = self.label_cluster_elements(input_dict, label, separator=separator,
                                                  label_position=label_position)
        output_dict.write(output_file, splited_values=True)

        return output_dict

    @staticmethod
    def extract_labels_from_cluster_elements(cluster_dict, separator="@", label_position="first"):
        labels_set = set()
        for cluster in cluster_dict:
            for element in cluster_dict[cluster]:
                label = element.split(separator)
                label = label[0] if label_position == "first" else label[-1]
                labels_set.add(label)
        return labels_set

    @staticmethod
    def replace_label(cluster_dict, syn_dict=None, old_separator="@", old_label_position="first", new_separator="@", new_label_position="first"):
        new_cluster_dict = SynDict()
        for cluster in cluster_dict:
            new_cluster_dict[cluster] = []
            for element in cluster_dict[cluster]:
                tmp = element.split(old_separator)
                if old_label_position == "first":
                    label = tmp[0]
                    element_id = old_separator.join(tmp[1:])
                else:
                    label = tmp[-1]
                    element_id = old_separator.join(tmp[:-1])

                if new_label_position == 'first':
                    new_cluster_dict[cluster].append("%s%s%s" % (syn_dict[label] if syn_dict else label, new_separator, element_id))
                else:
                    new_cluster_dict[cluster].append("%s%s%s" % (element_id, new_separator, syn_dict[label] if syn_dict else label))

        return new_cluster_dict

    def replace_label_from_file(self, input_file, output_file, syn_file_or_dict, old_separator="@",
                                old_label_position="first",
                                new_separator="@", new_label_position="first"):

        syn_dict = SynDict(filename=syn_file_or_dict, split_values=False) if isinstance(syn_file_or_dict, str) else SynDict(syn_file_or_dict)
        cluster_dict = SynDict(filename=input_file, split_values=True)
        new_cluster_dict = self.replace_label(cluster_dict, syn_dict=syn_dict, old_separator=old_separator,
                                              old_label_position=old_label_position, new_separator=new_separator,
                                              new_label_position=new_label_position)

        new_cluster_dict.write(output_file, splited_values=True)

        return new_cluster_dict

    @staticmethod
    def extract_single_copy_clusters(dict_of_cluster_dicts, label_elements=False, separator="@",
                                     label_position="first"):

        if label_position == "first":
            label_function = lambda s, label: "%s%s%s" % (label, separator, s)
        elif label_position == "last":
            label_function = lambda s, label: "%s%s%s" % (s, separator, label)

        sc_clusters_dict = SynDict()

        clusters_set = set()
        for group in dict_of_cluster_dicts:
            clusters_set = clusters_set | set(dict_of_cluster_dicts[group].keys())

        for cluster in clusters_set:
            for group in dict_of_cluster_dicts:
                if cluster not in dict_of_cluster_dicts[group]:
                    break
                if len(dict_of_cluster_dicts[group][cluster]) > 1:
                    break
            else:
                sc_clusters_dict[cluster] = []
                for group in dict_of_cluster_dicts:
                    if label_elements:
                        sc_clusters_dict[cluster].append(label_function(dict_of_cluster_dicts[group][cluster][0],
                                                                        group))
                    else:
                        sc_clusters_dict[cluster].append(dict_of_cluster_dicts[group][cluster][0])

        return sc_clusters_dict

    def extract_single_copy_clusters_from_files(self, list_of_cluster_files, output_file, label_elements=False, separator="@",
                                                label_position="first", function_to_convert_filename_to_label=None):
        dict_of_cluster_dicts = OrderedDict()
        for filename in list_of_cluster_files:
            if function_to_convert_filename_to_label:
                label = function_to_convert_filename_to_label(filename)
            else:
                label = self.split_filename(filename)[1]  # use basename as label

            dict_of_cluster_dicts[label] = SynDict()
            dict_of_cluster_dicts[label].read(filename, split_values=True, comments_prefix="#")

        sc_clusters_dict = self. extract_single_copy_clusters(dict_of_cluster_dicts, label_elements=label_elements,
                                                              separator=separator, label_position=label_position)

        sc_clusters_dict.write(output_file, splited_values=True)

        return sc_clusters_dict

    @staticmethod
    def remove_elements_by_ids(input_dict, black_list, mode="full"):
        filtered_dict = SynDict()

        for cluster_id in input_dict:
            elements = []
            for element_id in input_dict[cluster_id]:
                if mode == "full":
                    if element_id not in black_list:
                        elements.append(element_id)
                elif mode == "partial":
                    for black_id_part in black_list:
                        #print black_list
                        if black_id_part in element_id:
                            #print black_id_part, element_id
                            break
                    else:
                        #print "aaaa"
                        elements.append(element_id)
                else:
                    raise ValueError("Unknown mode for id comparison")
            if elements:
                filtered_dict[cluster_id] = elements

        return filtered_dict

    def remove_elements_by_ids_from_files(self, input_file, output_file, black_list_file, mode="full"):

        cluster_dict = SynDict(filename=input_file, split_values=True)
        black_list = IdList(filename=black_list_file)

        filtered_dict = self.remove_elements_by_ids(cluster_dict, black_list, mode=mode)

        filtered_dict.write(output_file, splited_values=True)

    @staticmethod
    def split_element_id(element_id, separator="@", label_position="first"):
        element_list = element_id.split(separator)
        if len(element_list) > 2:
            print("WARNING!!! Multiple separators in element id(%s)" % element_id)
        #    raise ValueError("Multiple separators in element id(%s)" % element_id)
        if len(element_list) < 2:
            raise ValueError("No separator was found in element id(%s)" % element_id)
        if label_position == "first":
            if len(element_list) > 2:
                return [element_list[0], separator.join(element_list[1:])]
            return element_list
        elif label_position == "last":
            if len(element_list) > 2:
                return [element_list[-1], separator.join(element_list[:-1])]
            return [element_list[1], element_list[0]]

    def extract_clusters_and_elements_by_labels(self, cluster_dict, label_list, separator="@", label_position="first"):
        filtered_dict = SynDict()

        for cluster_id in cluster_dict:
            for element_id in cluster_dict[cluster_id]:
                label, element = self.split_element_id(element_id, separator=separator,
                                                       label_position=label_position)
                if label in label_list:
                    if cluster_id in filtered_dict:
                        filtered_dict[cluster_id].append(element_id)
                    else:
                        filtered_dict[cluster_id] = [element_id]

        return filtered_dict

    def extract_clusters_and_elements_by_labels_from_files(self, cluster_file, label_file, output_file,
                                                           separator="@", label_position="first"):
        cluster_dict = SynDict(filename=cluster_file, split_values=True)
        label_list = IdList(filename=label_file) if isinstance(label_file, str) else label_file

        output_dict = self.extract_clusters_and_elements_by_labels(cluster_dict, label_list, separator=separator,
                                                                   label_position=label_position)

        output_dict.write(output_file, splited_values=True)

    def create_per_cluster_element_id_files(self, cluster_dict, output_directory):
        self.safe_mkdir(output_directory)

        for cluster_id in cluster_dict:
            cluster_element_id_list = IdList(cluster_dict[cluster_id])
            cluster_element_id_list.write("%s/%s.ids" % (output_directory, cluster_id))

    def create_per_cluster_element_id_files_from_file(self, cluster_file, output_directory,
                                                      cluster_column=0, element_column=1,
                                                      column_separator="\t", element_separator=","):

        cluster_dict = SynDict(filename=cluster_file, split_values=True, comments_prefix="#",
                               key_index=cluster_column, value_index=element_column,
                               separator=column_separator, values_separator=element_separator)

        self.create_per_cluster_element_id_files(cluster_dict, output_directory)

    def create_gvf_files_from_species_gene_fam_and_gene_GO_fam(self,
                                                               species_gene_fam_file,
                                                               gene_GO_fam_file,
                                                               output_directory):
        species_gene_dict = SynDict(filename=species_gene_fam_file, split_values=True)
        gene_GO_dict = SynDict(filename=gene_GO_fam_file, split_values=True)

        self.safe_mkdir(output_directory)

        for species in species_gene_dict:
            with open("%s/%s.gvf" % (output_directory, species), "w") as out_fd:
                for gene in species_gene_dict[species]:
                    if gene not in gene_GO_dict:
                        print("WARNING gene %s for species %s is absent in gene_GO file" % (gene, species))
                        continue
                    out_fd.write("%s\t%s\n" % (gene, "\t".join(gene_GO_dict[gene])))

    def cluster_sequence_names_by_id_fragment(self, seq_id_list, id_element_index, id_separator="_", output_prefix=None):
        cluster_dict = SynDict()
        skipped_id_list = IdList()

        for seq_id in seq_id_list:
            seq_id_splited = seq_id.split(id_separator)
            if id_element_index < len(seq_id_splited):
                if seq_id_list[id_element_index] in cluster_dict:
                    cluster_dict[seq_id_list[id_element_index]].append(seq_id)
                else:
                    cluster_dict[seq_id_list[id_element_index]] = [seq_id]
            else:
                skipped_id_list.append(seq_id)

        if output_prefix:
            cluster_dict.write("%s.seqid.clusters" % output_prefix, splited_values=True)
            skipped_id_list.write("%s.seqid.skipped.ids" % output_prefix)

        return cluster_dict

    def cluster_sequence_names_by_id_fragment_from_file(self, seq_id_file, id_element_index,
                                                        id_separator="_", output_prefix=None):

        seq_id_list = IdList(filename=seq_id_file)

        self.cluster_sequence_names_by_id_fragment(seq_id_list, id_element_index,
                                                   id_separator=id_separator, output_prefix=output_prefix)



#!/usr/bin/env python
import os
from copy import deepcopy
from Bio import SeqIO
from RouToolPa.Routines import NCBIRoutines

#from Routines.File import make_list_of_path_to_files
from RouToolPa.Collections.General import IdList, SynDict
from RouToolPa.Routines.SequenceCluster import SequenceClusterRoutines


class EggNOGRoutines(SequenceClusterRoutines):
    def __init__(self):
        SequenceClusterRoutines.__init__(self)
        self.members_tsv_family_column = 1
        self.members_tsv_sequence_id_column = 5

    @staticmethod
    def edit_profile_names_in_fam_file(input_fam_file, output_fam_file, comments_prefix="#"):
        if comments_prefix:
            comments_prefix_len = len(comments_prefix)

        with open(input_fam_file, "r") as in_fd:
            with open(output_fam_file, "w") as out_fd:
                for line in in_fd:
                    if comments_prefix:
                        if line[:comments_prefix_len] == comments_prefix:
                            out_fd.write(line)
                    line_list = line.split("\t")
                    fam_name = line_list[0].split(".")[1]
                    out_fd.write("%s\t%s" % (fam_name, line_list[1]))

    @staticmethod
    def convert_members_tsv_to_fam(input_file, output_file):
        cmd = "awk -F'\t' '{printf \"%%s\\t%%s\\n\",$2,$6 }' %s > %s" % (input_file, output_file)
        os.system(cmd)

    def extract_proteins_from_alignments(self, dir_with_alignments, output_dir):
        out_dir = self.check_path(output_dir)

        #print type(FileRoutines)

        input_files = self.make_list_of_path_to_files([dir_with_alignments] if isinstance(dir_with_alignments, str) else dir_with_alignments)

        self.safe_mkdir(out_dir)
        from RouToolPa.Routines import MultipleAlignmentRoutines
        for filename in input_files:
            filename_list = self.split_filename(filename)
            output_file = "%s%s%s" % (out_dir, filename_list[1], filename_list[2])
            MultipleAlignmentRoutines.extract_sequences_from_alignment(filename, output_file)

    def split_proteins_per_species(self, dir_with_proteins, output_dir, input_format="fasta", output_format="fasta"):
        #print type(FileRoutines)
        input_files = self.make_list_of_path_to_files([dir_with_proteins] if isinstance(dir_with_proteins, str) else dir_with_proteins)

        out_dir = self.check_path(output_dir)
        self.safe_mkdir(out_dir)

        protein_dict = SeqIO.index_db("temp.idx", input_files, format=input_format)

        syn_dict = SynDict()

        for protein_id in protein_dict:
            taxa_id = protein_id.split(".")[0]
            # pep_id = ".".join(tmp_list[1:])
            if taxa_id not in syn_dict:
                syn_dict[taxa_id] = []
            syn_dict[taxa_id].append(protein_id)

        def renamed_records_generator(record_dict, taxa_id):
            for record_id in syn_dict[taxa_id]:
                record = deepcopy(record_dict[record_id])
                #print(record)
                record.id = ".".join(record_id.split(".")[1:])
                yield record

        for taxa_id in syn_dict:
            out_file = "%s%s.pep" % (out_dir, taxa_id)
            SeqIO.write(renamed_records_generator(protein_dict, taxa_id), out_file, format=output_format)

    def get_species_from_eggnog_tsv(self, eggnog_tsv, output_prefix, email=None):

        cluster_dict = SynDict(filename=eggnog_tsv,
                               key_index=self.members_tsv_family_column,
                               value_index=self.members_tsv_sequence_id_column,
                               split_values=True)

        species_ids = self.extract_labels_from_cluster_elements(cluster_dict, separator=".", label_position="first")
        print("Input species ids: %s" % " ".join(species_ids))

        if not email:
            species = species_ids
        else:
            species = NCBIRoutines.get_taxonomy(species_ids, "%s.species.taxonomy" % output_prefix,
                                                email, input_type="id")

        species.write("%s.species" % output_prefix, splited_values=True)

        for species_id in species:
            for i in range(0, len(species[species_id])):
                species[species_id][i] = species[species_id][i].lower().replace(" ", "_")

        species.write("%s.replaced_spaces.species" % output_prefix, splited_values=True)

    def extract_eggnog_fam_by_protein_syn_dict(self, eggnog_fam_dict, protein_syn_dict, output_prefix=None, species_id=None):

        extracted_families = SynDict()
        common_protein_names_to_families_dict = SynDict()
        common_names_to_eggnog_proteins_syn_dict = SynDict()

        not_found_proteins_common_names = IdList()

        transposed_eggnog_fam_dict = eggnog_fam_dict.exchange_key_and_value()

        for common_protein_name in protein_syn_dict:
            not_found = True
            for protein_id in protein_syn_dict[common_protein_name]:
                extended_protein_id = protein_id if species_id is None else species_id + "." + protein_id
                if extended_protein_id in transposed_eggnog_fam_dict:
                    not_found = False
                    if common_protein_name not in common_protein_names_to_families_dict:
                        common_protein_names_to_families_dict[common_protein_name] = [transposed_eggnog_fam_dict[extended_protein_id][0]]
                        common_names_to_eggnog_proteins_syn_dict[common_protein_name] = [extended_protein_id]
                    else:
                        common_protein_names_to_families_dict[common_protein_name].append(transposed_eggnog_fam_dict[extended_protein_id][0])
                        common_names_to_eggnog_proteins_syn_dict[common_protein_name].append(extended_protein_id)
                    if transposed_eggnog_fam_dict[extended_protein_id][0] not in extracted_families:
                        extracted_families[transposed_eggnog_fam_dict[extended_protein_id][0]] = eggnog_fam_dict[transposed_eggnog_fam_dict[extended_protein_id][0]]

            if not_found:
                not_found_proteins_common_names.append(common_protein_name)

        if output_prefix:
            extracted_families.write(filename="%s.extracted_families.fam" % output_prefix, splited_values=True)
            common_protein_names_to_families_dict.write(filename="%s.common_protein_names_to_families.correspondence" % output_prefix, splited_values=True)
            common_names_to_eggnog_proteins_syn_dict.write(filename="%s.common_protein_names_to_eggnog_proteins.correspondence" % output_prefix, splited_values=True)
            not_found_proteins_common_names.write(filename="%s.not_found.common_names" % output_prefix)

            #print common_names_to_eggnog_proteins_syn_dict
            #print common_protein_names_to_families_dict
        return extracted_families, common_protein_names_to_families_dict, \
               common_names_to_eggnog_proteins_syn_dict, not_found_proteins_common_names




#!/usr/bin/env python
import os
import shutil
from RouToolPa.Tools.Abstract import Tool



class BLAT(Tool):

    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "blat", path=path, max_threads=max_threads)

    def parse_options(self, database_type=None, query_type=None, add_header=True):

        options = ""
        options += " -t=%s" % database_type if database_type else ""
        options += " -q=%s" % query_type if query_type else ""
        options += " -noHead" if not add_header else ""

        return options

    def parallel_align(self, database, query_fasta, output, split_dir="splited_input/",
                       splited_output_dir="splited_output_dir/",
                       database_type=None, query_type=None, add_header=True,
                       threads=None, remove_tmp_dirs=True,
                       async_run=False, external_process_pool=None):
        splited_dir = self.check_path(split_dir)
        splited_out_dir = self.check_path(splited_output_dir)
        self.safe_mkdir(splited_dir)
        self.safe_mkdir(splited_out_dir)

        common_options = self.parse_options(database_type=database_type,
                                            query_type=query_type,
                                            add_header=add_header)

        number_of_files = 5 * threads if threads else 5 * self.threads
        self.split_fasta(query_fasta, splited_dir, num_of_files=number_of_files)

        options_list = []
        input_list_of_files = sorted(os.listdir(splited_dir))
        output_file_list = []
        for filename in input_list_of_files:
            query = "%s/%s" % (splited_dir, filename)
            output_file = "%s/%s.out" % (splited_out_dir, filename)
            options = "%s" % database
            options += " %s" % query
            options += " %s" % common_options
            options += " %s" % output_file

            options_list.append(options)
            output_file_list.append(output_file)

        self.parallel_execute(options_list, cmd="blat", threads=threads, async_run=async_run,
                              external_process_pool=external_process_pool)
        if add_header:
            with open(output, "w") as out_fd:
                pass
            os.system("cat %s > %s" % (output_file_list[0], output))
            for filename in output_file_list[1:]:
                sed_string = "sed -n 6,$p %s > %s" % (filename, output)
                os.system(sed_string)
        else:
            cat_string = "cat %s > %s" % ("\t".join(output_file_list), output)
            os.system(cat_string)

        if remove_tmp_dirs:
            shutil.rmtree(splited_dir, ignore_errors=True)
            shutil.rmtree(splited_output_dir, ignore_errors=True)

    def align_pep_for_scaffolding(self, database, query_fasta, output, split_dir="splited_input/",
                                  splited_output_dir="splited_output_dir/",
                                  threads=None, remove_tmp_dirs=True,
                                  async_run=False, external_process_pool=None):

        self.parallel_align(database, query_fasta, output, split_dir=split_dir,
                            splited_output_dir=splited_output_dir,
                            database_type="dnax", query_type="prot", add_header=False,
                            threads=threads, remove_tmp_dirs=remove_tmp_dirs,
                            async_run=async_run, external_process_pool=external_process_pool)

    def align_transcripts_for_scaffolding(self, database, query_fasta, output, split_dir="splited_input/",
                                          splited_output_dir="splited_output_dir/",
                                          threads=None, remove_tmp_dirs=True,
                                          async_run=False, external_process_pool=None):

        self.parallel_align(database, query_fasta, output, split_dir=split_dir,
                            splited_output_dir=splited_output_dir,
                            database_type="dna", query_type="dna", add_header=False,
                            threads=threads, remove_tmp_dirs=remove_tmp_dirs,
                            async_run=async_run, external_process_pool=external_process_pool)


#!/usr/bin/env python

import os
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio.Alphabet import IUPAC
from RouToolPa.Routines.Sequence import SequenceRoutines


class MtDNARoutines(SequenceRoutines):

    def __init__(self):
        SequenceRoutines.__init__(self)
        self.mithochondrion_synonym_table = [
                                            ["12S_rRNA",
                                             "12S rRNA",
                                             "12S ribosomal RNA",
                                             "small subunit ribosomal RNA",
                                             "s-rRNA", "s-rRNA; 12S ribosomal RNA",
                                             "small ribosomal RNA subunit RNA",
                                             "12S ribosomal RNA",
                                             "12S ribosomal RNA subunit",
                                             "12S rivbosomal RNA",
                                             "12S ribosamal RNA",
                                             "l2S ribosomal RNA",
                                             "12 ribosomal RNA",
                                             "12S ribosormal RNA,"
                                             "12 rRNA",
                                             "s-RNA"],
                                            ["16S_rRNA",
                                             "16S rRNA",
                                             "16S ribosomal RNA",
                                             "large subunit ribosomal RNA",
                                             "l-rRNA",
                                             "l-rRNA; 16S ribosomal RNA",
                                             "large ribosomal RNA subunit RNA",
                                             "16S ribosomal RNA",
                                             "16S ribosomal RNA subunit",
                                             "16S rivbosomal RNA",
                                             "16S ribosamal RNA",
                                             "l6S ribosomal RNA",
                                             "16 ribosomal RNA",
                                             "16S ribosormal RNA",
                                             "16 rRNA",
                                             "l-RNA"],
                                            ["ATP6",
                                             "atp6",
                                             "ATPase6",
                                             "ATPase 6",
                                             "ATPase subunit 6",
                                             "ATP synthase F0 subunit 6",
                                             "ATP synthetase F0 subunit 6",
                                             "ATP synthase subunit 6"
                                             "ATPase subunits 6",
                                             "adenosine triphosphatase subunit 6",
                                             "ATPase subunit-6"],
                                            ["ATP8",
                                             "atp8",
                                             "ATPase8",
                                             "ATPase 8",
                                             "ATPase subunit 8",
                                             "ATP synthase F0 subunit 8",
                                             "ATP synthetase F0 subunit 8",
                                             "ATP synthase subunit 8",
                                             "ATPase subunits 8",
                                             "adenosine triphosphatase subunit 8",
                                             "adenosine triphosphate subunit 8",
                                             "ATPase subunit-8"],
                                            ["COX1", "COXI",
                                             "cytochrome c oxidase subunit 1",
                                             "cytochrome c oxidase subunit I",
                                             "Cytochrome c oxidase subunit 1",
                                             "cytochrome oxidase subunit I",
                                             "chytochrome c oxidase subunit I",
                                             "COI",
                                             "CO1", "CO 1", "CO I", "coi",
                                             "product: cytochrome c oxidase subunit I",
                                             "cytochrome oxidase subunit 1"],
                                            ["COX2", "COXII",
                                             "cytochrome c oxidase subunit 2",
                                             "cytochrome c oxidase subunit II",
                                             "Cytochrome c oxidase subunit 2",
                                             "cytochrome oxidase subunit II",
                                             "chytochrome c oxidase subunit II",
                                             "COII",
                                             "CO2", "CO 2", "CO II", "coii",
                                             "cytochrome oxidase subunit 2"],
                                            ["COX3", "COXIII",
                                             "cytochrome c oxidase subunit 3",
                                             "cytochrome c oxidase subunit III",
                                             "Cytochrome c oxidase subunit 3",
                                             "cytochrome oxidase subunit III",
                                             "chytochrome c oxidase subunit III",
                                             "COIII",
                                             "CO3", "CO 3", "CO III", "coiii",
                                             "cytochrome oxidase subunit 3"],
                                            ["CYTB", "cytochrome b", "Cytochrome b", "cytb", "Cytb", "Cyt b",
                                             "Cytochrome b apoenzyme",
                                             "cytochrome b apoenzyme",
                                             "cytochrome b; TAA stop codon appears afterpolyadenylation"
                                             ],
                                            ["ND1", "nd1", "nd 1", "ND 1", "Nd 1",
                                             "NADH dehydrogenase subunit 1",
                                             "NADH hydrogenase subunit 1",
                                             "subunit 1 of the NADH ubiquinone oxidoreductase complex",
                                             "NADH-1", "NADH1"],
                                            ["ND2", "nd2", "nd 2", "ND 2", "Nd 2",
                                             "NADH dehydrogenase subunit 2",
                                             "NADH hydrogenase subunit 2",
                                             "subunit 2 of the NADH ubiquinone oxidoreductase complex",
                                             "#NADH dehydrogenase subunit 2",
                                             "NADH-2", "NADH2"],
                                            ["ND3", "nd3", "nd 3", "ND 3", "Nd 3",
                                             "NADH dehydrogenase subunit 3",
                                             "NADH hydrogenase subunit 3",
                                             "subunit 3 of the NADH ubiquinone oxidoreductase complex",
                                             "NADH-3", "NADH3"],
                                            ["ND4", "nd4", "nd 4", "ND 4", "Nd 4",
                                             "NADH dehydrogenase subunit 4",
                                             "NADH hydrogenase subunit 4",
                                             "subunit 4 of the NADH ubiquinone oxidoreductase complex",
                                             "NADH-4", "NADH4"],
                                            ["ND4L", "nd4l", "nd 4l", "ND 4l", "Nd 4l",
                                             "NADH dehydrogenase subunit 4L",
                                             "NADH hydrogenase subunit 4L",
                                             "NADH-4L", "NADH4L"],
                                            ["ND5", "nd5", "nd 5", "ND 5", "Nd 5",
                                             "NADH dehydrogenase subunit 5",
                                             "NADH hydrogenase subunit 5",
                                             "subunit 5 of the NADH ubiquinone oxidoreductase complex",
                                             "NADH-5", "NADH5"],
                                            ["ND6", "nd6", "nd 6", "ND 6", "Nd 6",
                                             "NADH dehydrogenase subunit 6",
                                             "NADH hydrogenase subunit 6",
                                             "subunit 6 of the NADH ubiquinone oxidoreductase complex",
                                             "NADH-6", "NADH6",
                                             "NADH dehydrogenase subunit-6"],
                                            ["tRNA-Val"], ["tRNA-Leu"], ["tRNA-Phe"], ["tRNA-Pro"], ["tRNA-Thr"],
                                            ["tRNA-Glu"], ["tRNA-Ser"], ["tRNA-His"], ["tRNA-Arg"], ["tRNA-Gly"],
                                            ["tRNA-Lys"], ["tRNA-Asp"], ["tRNA-Tyr"], ["tRNA-Cys"], ["tRNA-Asn"],
                                            ["tRNA-Ala"], ["tRNA-Trp"], ["tRNA-Met"], ["tRNA-Ile"], ["tRNA-Gln"]
                                            ]
        self.protein_gene_list = [
                                  "ATP6",
                                  "ATP8",
                                  "COX1",
                                  "COX2",
                                  "COX3",
                                  "CYTB",
                                  "ND1",
                                  "ND2",
                                  "ND3",
                                  "ND4L",
                                  "ND4",
                                  "ND5",
                                  "ND6",
                                  ]

        self.rRNA_gene_list = [
                               "12S_rRNA",
                               "16S_rRNA"
                              ]

    @staticmethod
    def modify_record(input_record, record_id, record_name, record_description):
        modified_record = input_record
        modified_record.id = record_id
        modified_record.name = record_name
        modified_record.description = record_description
        return modified_record

    def split_mitochondrion_genome_by_genes(self, record_dict, black_list=[]):

        region_dict = {}
        protein_dict = {}

        def check_for_synonym(gene_name, gene_synonym_table):
            for gene_synonym_list in gene_synonym_table:
                if gene_name in gene_synonym_list:
                    correct_gene_name = gene_synonym_list[0]
                    break
            else:
                correct_gene_name = None
            return correct_gene_name

        def get_feature_gene(feature):
            feature_gene_list = []
            if "product" in feature.qualifiers:
                feature_gene_list.append(check_for_synonym(feature.qualifiers["product"][0], self.mithochondrion_synonym_table))
            if "gene" in feature.qualifiers:
                feature_gene_list.append(check_for_synonym(feature.qualifiers["gene"][0], self.mithochondrion_synonym_table))

            if ("gene" not in feature.qualifiers) and ("product" not in feature.qualifiers):
                if "note" in feature.qualifiers:
                    feature_gene_list.append(check_for_synonym(feature.qualifiers["note"][0], self.mithochondrion_synonym_table))
            """
            if "note" in feature.qualifiers:
                feature_gene_list.append(check_for_synonym(feature.qualifiers["note"], mithochondrion_synonym_table))
            """
            for gene_name in feature_gene_list:
                if gene_name:
                    return gene_name
            return None

        nucleotide_exception_records = []
        protein_exception_records = []
        for record_id in record_dict:
            #print record_id
            if record_id in black_list:
                continue
            nucleotide_record_features = {}
            protein_record_features = {}
            for gene in self.rRNA_gene_list + self.protein_gene_list:
                nucleotide_record_features[gene] = 0
            for gene in self.protein_gene_list:
                protein_record_features[gene] = 0
            #print("\t%s" % record_id)
            for feature in record_dict[record_id].features:
                #print("\t%s" % record_id)
                #print(feature)
                if feature.type == "source" or feature.type == "gene" or feature.type == "misc_feature":
                    continue
                elif feature.type == "rRNA":
                    name = get_feature_gene(feature)
                    new_record = self.modify_record(feature.extract(record_dict[record_id]),
                                                    record_id,
                                                    name,
                                                    "")
                    if name:
                        nucleotide_record_features[name] += 1
                        #print(record_id, name, nucleotide_record_features)
                        if name not in region_dict:
                            region_dict[name] = [new_record]
                        else:
                            region_dict[name].append(new_record)
                    else:
                        print("Unrecognized rRNA feature\t%s" % (record_id))
                elif feature.type == "tRNA":
                    """
                    name = get_feature_gene(feature)
                    description = ""
                    if "note" in feature.qualifiers:
                        description = feature.qualifiers["note"][0]
                    new_record = modify_record(feature.extract(record_dict[record_id]),
                                               record_id,
                                               name,
                                               description)
                    if name:
                        if name not in region_dict:
                            region_dict[name] = [new_record]
                        else:
                            region_dict[name].append(new_record)
                    else:
                        print("Unrecognized tRNA feature\t%s\n %s" % (record_id, str(feature)))
                    """
                    description = ""
                    name = ""
                    if "note" in feature.qualifiers:
                        description = feature.qualifiers["note"][0]
                    if "product" in feature.qualifiers:
                        name = feature.qualifiers["product"][0]
                    new_record = self.modify_record(feature.extract(record_dict[record_id]),
                                                    record_id,
                                                    name,
                                                    description)
                    if "product" in feature.qualifiers:
                        if feature.qualifiers["product"][0] not in region_dict:
                            region_dict[feature.qualifiers["product"][0]] = [new_record]
                        else:
                            region_dict[feature.qualifiers["product"][0]].append(new_record)
                    else:
                        print("Unrecognized tRNA feature\t%s" % (record_id))

                elif feature.type == "CDS":
                    name = get_feature_gene(feature)
                    if name:
                        nucleotide_record_features[name] += 1
                        new_region_record = self.modify_record(feature.extract(record_dict[record_id]),
                                                               record_id,
                                                               name,
                                                               "")
                        if "translation" not in feature.qualifiers:
                            print("Nontranslated CDS feature\t%s\t%s" % (record_id, str(feature)))
                            continue
                            #print(feature)

                        new_protein_record = SeqRecord(seq=Seq(feature.qualifiers["translation"][0], alphabet=IUPAC.protein),
                                                    id=record_id,
                                                    name=name,
                                                    description="")
                        protein_record_features[name] += 1
                        #print(record_id, feature, new_protein_record)
                        if name not in region_dict:
                            region_dict[name] = [new_region_record]
                            protein_dict[name] = [new_protein_record]
                        else:
                            region_dict[name].append(new_region_record)
                            protein_dict[name].append(new_protein_record)
                    else:
                        print("Unrecognized CDS feature\t%s\t%s" % (record_id, str(feature)))
                        #print("\n")
                        #print(feature.qualifiers["product"][0])
                        #print(feature.qualifiers["gene"])
                        #print(feature.qualifiers["product"])
                    #print(feature.qualifiers["translation"][0])
                    #pass
                    #print(feature.qualifiers["gene"])
                elif feature.type == "D-loop":
                    new_record = self.modify_record(feature.extract(record_dict[record_id]),
                                                    record_id,
                                                    "D-loop",
                                                    "")
                    if "D-loop" not in region_dict:
                        region_dict["D-loop"] = [new_record]
                    else:
                        region_dict["D-loop"].append(new_record)
                    #print(feature.extract(record_dict[record_id]))
            for key in nucleotide_record_features:
                #print key
                #print nucleotide_record_features[key]
                if nucleotide_record_features[key] != 1:
                    print("Suspicious record %s. Duplication or absense genes" % record_id)
                    #print(nucleotide_record_features)
                    nucleotide_exception_records.append(record_id)
                    break
            for key in protein_record_features:
                if protein_record_features[key] != 1:
                    print("Suspicious record %s. Duplication or absense genes" % record_id)
                    #print(protein_record_features)
                    protein_exception_records.append(record_id)
                    break
        nucleotide_exception_records = set(nucleotide_exception_records)
        protein_exception_records = set(protein_exception_records)
        #print nucleotide_exception_records
        #print protein_exception_records
        sudpicious_records = nucleotide_exception_records | protein_exception_records
        print("Totally suspicious records : %i" % len(sudpicious_records))
        print("Suspicious records: %s " % ",".join(sudpicious_records))

        with open("suspicious_records.t", "w") as fd_sus:
            fd_sus.write("#id\n" + "\n".join(sudpicious_records))

        def write_data(data_dict, data_type, white_list=None):
            os.system("mkdir -p %s" % data_type)
            os.system("mkdir -p %s/fasta" % data_type)
            os.system("mkdir -p %s/genbank" % data_type)
            for entry in data_dict:
                fd_fasta = open("%s/fasta/" % data_type + entry + ".fasta", "w")
                fd_genbank = open("%s/genbank/" % data_type + entry + ".gb", "w")
                if white_list:
                    i = 0
                    number_of_records = len(data_dict[entry])
                    while i < number_of_records:
                        if data_dict[entry][i].id not in white_list:
                            del(data_dict[entry][i])
                            number_of_records -= 1
                            continue
                        i += 1
                SeqIO.write(data_dict[entry], fd_fasta, "fasta")
                SeqIO.write(data_dict[entry], fd_genbank, "genbank")
                fd_fasta.close()
                fd_genbank.close()

        good_ids = set(record_dict.keys()) - nucleotide_exception_records - protein_exception_records
        """
        regions_set = []
        protein_set = []
        if check_presense_coding_rRNA_genes:
            #regions_list = protein_gene_list + rRNA_gene_list
            for region in protein_gene_list:
                temp_set = []
                for record in region_dict[region]:
                    temp_set.append(record.id)
                if not regions_set:
                    regions_set = set(temp_set)
                    protein_set = set(temp_set)
                    continue
                regions_set = regions_set & set(temp_set)
                protein_set = protein_set & set(temp_set)
            for region in rRNA_gene_list:
                for record in region_dict[region]:
                    temp_set.append(record.id)
                regions_set = regions_set & set(region_dict[region])
        """
        write_data(region_dict, "nucleotide", white_list=good_ids)
        write_data(protein_dict, "protein", white_list=good_ids)
        tested_nucleotide_dict = {}
        tested_protein_dict = {}
        for gene in self.protein_gene_list + self.rRNA_gene_list:
            try:
                tested_nucleotide_dict[gene] = list(SeqIO.index("nucleotide/genbank/%s.gb" % gene, "genbank").keys())
            except ValueError:
                print("Error. Possible duplication of %s" % gene)
        for gene in self.protein_gene_list:
            try:
                tested_protein_dict[gene] = list(SeqIO.index("protein/genbank/%s.gb" % gene, "genbank").keys())
            except ValueError:
                print("Error. Possible duplication of %s in one of genomes" % gene)

        self.test_splited_genes(record_dict, tested_nucleotide_dict, output_filename="nucleotide_check_absence.t")
        self.test_splited_genes(record_dict, tested_protein_dict, output_filename="protein_check_absence.t")

    @staticmethod
    def test_splited_genes(record_dict, tested_record_dict, output_filename="check_gene_absence.t"):
        #tested_record_dict - key = gene name , element - list of record_ids
        absent_ids_dict = {}
        for key in tested_record_dict:
            absent_ids_dict[key] = set(record_dict.keys()) - set(tested_record_dict[key])
        with open(output_filename, "w") as fd:
            fd.write("#gene\tabsent\tids\n")
            for key in absent_ids_dict:
                fd.write("%s\t%i\t%s\n" % (key, len(absent_ids_dict[key]), ",".join(absent_ids_dict[key])))


__author__ = 'mahajrod'
import pprint
from BCBio import GFF
from BCBio.GFF import GFFExaminer
from Bio.SeqRecord import SeqRecord
from Bio import SeqIO
from RouToolPa.Tools.Abstract import Tool


#TODO: finish code
def examine_gff(gff_file):
    examiner = GFFExaminer()
    in_handle = open(gff_file)
    pprint.pprint(examiner.available_limits(in_handle))
    print("")
    in_handle.close()


def parse_gff(gff_file, limit_info={}):
    examine_gff(gff_file)
    file_handler = open(gff_file)
    if limit_info:
        gff_ann = list(GFF.parse(file_handler, limit_info=limit_info))
    else:
        gff_ann = list(GFF.parse(file_handler))
    file_handler.close()
    #print gff_ann
    return gff_ann


def make_glimmer_training_data(seq_file, seq_index, gff_file, out_transcript_file,
                               out_exon_coordinates, seq_filetype="fasta"):
    fasta_dict = SeqIO.index_db(seq_index, [seq_file], seq_filetype)

    def generator(gff_file, fasta_dict, exon_fd):
        with open(gff_file, "r") as gff_fd:
            for record in GFF.parse(gff_fd, target_lines=100000):
                for feature in record.features:
                    #print (feature.type)
                    if feature.type == "transcript":
                        print(feature)
                        print(feature.sub_features)
                        exon_fd.write(feature.id + "\n")
                        exon_fd.write(str(feature.location))
                        exon_fd.write("\n")
                        exon_fd.write(str(feature.sub_features) + "\n")

                    if feature.type == "gene":
                        feature_record = feature.extract(fasta_dict[record.id])
                        feature_record.id = feature.qualifiers["gene_id"][0]
                        feature_record.description = ""
                        yield feature_record
    exon_fd = open(out_exon_coordinates, "w")
    SeqIO.write(generator(gff_file, fasta_dict, exon_fd), out_transcript_file, "fasta")
    exon_fd.close()


if __name__ == "__main__":

    make_glimmer_training_data("/run/media/mahajrod/Data/ensemble_75/fasta/takifugu_rubripes/dna/Takifugu_rubripes.FUGU4.75.dna.toplevel.fa",
                               "/run/media/mahajrod/Data/ensemble_75/fasta/takifugu_rubripes/dna/Takifugu_rubripes.FUGU4.75.dna.toplevel.index",
                               "/run/media/mahajrod/Data/ensemble_75/gtf/takifugu_rubripes/Takifugu_rubripes.FUGU4.75.gtf",
                               "/run/media/mahajrod/Data/ensemble_75/fasta/takifugu_rubripes/dna/Takifugu_rubripes.FUGU4.75.transcripts.fa",
                               "/run/media/mahajrod/Data/ensemble_75/fasta/takifugu_rubripes/dna/Takifugu_rubripes.FUGU4.75.exons.coordinates",
                               seq_filetype="fasta")



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os

from aindex import *

from RouToolPa.Tools.Abstract import Tool

from RouToolPa.Tools.EMPHF import EMPHF
from RouToolPa.Tools.Kmers import Jellyfish


class AIndex(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "aindex", path=path, max_threads=max_threads)

    def compute_index(self, counts_file, pf_file, index_prefix,
                      full_hash=True):
        """

        :param counts_file:
        :param pf_file:
        :param index_prefix:
        :param full_hash:
        :return:

        creates following files: <index_prefix>.kmers.bin
                                 <index_prefix>.tf.bin
        """
        options = " %s" % counts_file
        options += " %s" % pf_file
        options += " %s" % index_prefix
        options += " %i" % self.threads
        options += " 0" if full_hash else ""

        self.execute(options=options, cmd="compute_index.exe")

    def compute_reads(self, forward_file, reverse_file, output, filetype="fastq"):
        """

        :param forward_file:
        :param reverse_file:
        :param output:
        :param filetype:
        :return:

        creates <output_file>
        """
        options = " %s" % forward_file
        options += " %s" % reverse_file
        options += " %s" % filetype
        options += " %s" % output

        self.execute(options=options, cmd="compute_reads.exe")

    def compute_aindex(self, reads_file, pf_file, freq_index_prefix,
                       read_index_prefix, kmer_length, index_scheme_file):

        options = " %s" % reads_file
        options += " %s " % pf_file
        options += " %s " % freq_index_prefix
        options += " %s " % read_index_prefix
        options += " %i " % self.threads
        options += " %i " % kmer_length
        options += " %s " % index_scheme_file

        self.execute(options=options, cmd="compute_aindex.exe")

    def create_index_from_jf(self, jf_db, kmer_length, output_prefix=None,
                             lower_count=1, upper_count=None,
                             forward_file=None, reverse_file=None,
                             filetype="fastq",
                             create_aindex=False):

        out_pref = output_prefix if output_prefix else jf_db

        #if lower_count:
        #    out_pref = "%s.l%i" % (out_pref, lower_count)
        #if upper_count:
        #    out_pref = "%s.x%i" % (out_pref, upper_count)

        counts_file = "%s.counts" % out_pref
        kmer_file = "%s.kmers" % out_pref
        pf_file = "%s.pf" % out_pref
        reads_file = "%s.reads" % out_pref
        index_scheme_file = "%s.tf.bin" % out_pref

        print("Extracting kmers from jf database...")
        Jellyfish.dump_kmers(jf_db, out_pref, lower_count=lower_count,
                             upper_count=upper_count)
        print("Creating index...")
        EMPHF.compute_mphf_seq(kmer_file, pf_file)
        self.compute_index(counts_file, pf_file, out_pref)

        if create_aindex:
            print("Creating AIndex...")
            self.compute_reads(forward_file, reverse_file, reads_file, filetype=filetype)
            self.compute_aindex(reads_file, pf_file, out_pref,
                                out_pref, kmer_length, index_scheme_file)

    def create_index_from_fastq(self, forward_file, reverse_file,
                                kmer_length, output_prefix,
                                lower_count=1, upper_count=None,
                                filetype="fastq",
                                create_aindex=False,
                                hash_size="10G"):

        out_pref = "%s.k%i" % (output_prefix, kmer_length)

        if lower_count:
            out_pref = "%s.l%i" % (out_pref, lower_count)
        if upper_count:
            out_pref = "%s.x%i" % (out_pref, upper_count)

        jf_db = "%s.jf" % out_pref

        Jellyfish.threads = self.threads
        print("Creating jf database...")
        Jellyfish.count([forward_file, reverse_file], jf_db,
                        kmer_length=kmer_length, hash_size=hash_size,
                        count_both_strands=True,
                        lower_count=lower_count, upper_count=upper_count)

        self.create_index_from_jf(jf_db, kmer_length,
                                  output_prefix=out_pref,
                                  lower_count=lower_count,
                                  upper_count=upper_count,
                                  forward_file=forward_file,
                                  reverse_file=reverse_file,
                                  filetype=filetype,
                                  create_aindex=create_aindex)


if __name__ == "__main__":
    pass



__author__ = 'mahajrod'

from RouToolPa.Routines.GO import GORoutines
from RouToolPa.Routines.VCF import VCFRoutines
from RouToolPa.Routines.Math import MathRoutines
from RouToolPa.Routines.File import FileRoutines
from RouToolPa.Routines.Tree import TreeRoutines
from RouToolPa.Routines.NCBI import NCBIRoutines
from RouToolPa.Routines.Math import SmoothRoutines
from RouToolPa.Routines.MtDNA import MtDNARoutines
from RouToolPa.Routines.Fastq import FastQRoutines
from RouToolPa.Routines.Primer import PrimerRoutines
from RouToolPa.Routines.EggNOG import EggNOGRoutines
from RouToolPa.Routines.Project import ProjectRoutines
from RouToolPa.Routines.Ensembl import EnsemblRoutines
from RouToolPa.Routines.TreeFam import TreeFamRoutines
from RouToolPa.Routines.Drawing import DrawingRoutines
from RouToolPa.Routines.Sequence import SequenceRoutines
from RouToolPa.Routines.Evolution import EvolutionRoutines
from RouToolPa.Routines.Alignment import AlignmentRoutines
from RouToolPa.Routines.Expression import ExpressionRoutines
from RouToolPa.Routines.Matplotlib import MatplotlibRoutines
from RouToolPa.Routines.Annotations import AnnotationsRoutines
from RouToolPa.Routines.Phylogenetics import PhylogeneticsRoutines
from RouToolPa.Routines.SequenceCluster import SequenceClusterRoutines
from RouToolPa.Routines.MultipleAlignment import MultipleAlignmentRoutines


GORoutines = GORoutines()
VCFRoutines = VCFRoutines()
MathRoutines = MathRoutines()
FileRoutines = FileRoutines()
TreeRoutines = TreeRoutines()
NCBIRoutines = NCBIRoutines()
MtDNARoutines = MtDNARoutines()
FastQRoutines = FastQRoutines()
SmoothRoutines = SmoothRoutines()
PrimerRoutines = PrimerRoutines()
EggNOGRoutines = EggNOGRoutines()
ProjectRoutines = ProjectRoutines()
EnsemblRoutines = EnsemblRoutines()
TreeFamRoutines = TreeFamRoutines()
DrawingRoutines = DrawingRoutines()
SequenceRoutines = SequenceRoutines()
EvolutionRoutines = EvolutionRoutines()
AlignmentRoutines = AlignmentRoutines()
ExpressionRoutines = ExpressionRoutines()
MatplotlibRoutines = MatplotlibRoutines()
AnnotationsRoutines = AnnotationsRoutines()
PhylogeneticsRoutines = PhylogeneticsRoutines()
SequenceClusterRoutines = SequenceClusterRoutines()
MultipleAlignmentRoutines = MultipleAlignmentRoutines()



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'
import os
from RouToolPa.Tools.Abstract import Tool


class SortVcf4(Tool):

    def __init__(self, max_threads=4,  max_memory="4g"):
        Tool.__init__(self, "gatk SortVcf", max_threads=max_threads,
                      max_memory=max_memory)

    def sort_vcf(self, input_file, output_file, seq_dict=None,
                 handling_mode="local",
                 max_memory_per_node=None,
                 job_name=None,
                 log_prefix=None,
                 error_log_prefix=None,
                 modules_list=None,
                 environment_variables_dict=None,
                 max_running_time=None):

        input_file_list = self.make_list_of_path_to_files_from_string(input_file) if isinstance(input_file, str) else self.make_list_of_path_to_files(input_file)

        options = ""

        for filename in input_file_list:
            options += " -I %s" % filename

        options += " -O %s" % output_file
        options += " -SD %s" % seq_dict if seq_dict else ""

        if handling_mode == 'local':
            self.execute(options=options,
                         cmd=("gatk --java-options -Xmx%s SortVcf" % self.max_memory) if self.max_memory else None)
        elif handling_mode == 'slurm':
            self.timelog = None
            slurm_cmd = self.execute(options=options, generate_cmd_string_only=True,
                                     cmd=("gatk --java-options -Xmx%s SortVcf" % self.max_memory) if self.max_memory else None)

            job_id = self.slurm_run_job(job_name, log_prefix, slurm_cmd, error_log_prefix,
                                        "".join(self.split_filename(output_file)[:2]) + ".slurm",
                                        modules_list=modules_list,
                                        environment_variables_dict=environment_variables_dict,
                                        max_running_time=max_running_time,
                                        max_memory_per_node=max_memory_per_node)

            return job_id

if __name__ == "__main__":
    pass
